@@ -7,7 +7,6 @@ import numpy as np
 import matplotlib.pyplot as plt
 
 
-
 def getAtomsToMove(i,j,j_,ToBeMove,neighbors,ring=False):
     ToBeMove.append(j_)
     for n in neighbors[j_]:
@@ -94,12 +93,17 @@ def get_zmat_variable(i,j,k,l,positions):
     vj   = np.cross(ulk,ujk)
     rj   = np.sqrt(np.sum(np.square(vj)))
     uj   = vj/rj
-
+    
     cos_ = np.dot(ui,uj)
     vk   = np.cross(ui,uj)
-    uk   = vk/np.sqrt(np.sum(np.square(vk)))
-    s    = np.dot(uk,ujk)
-    tor  = s*np.arccos(cos_)*180.0/3.141593
+    rk   = np.sqrt(np.sum(np.square(vk)))
+    if rk==0.0:
+       tor = 0.0
+    else:
+       uk   = vk/rk
+       # print(ui,uj,cos_,vk,uk)
+       s    = np.dot(uk,ujk)
+       tor  = s*np.arccos(cos_)*180.0/3.141593
     return rij,ang,tor
 
 
@@ -379,7 +383,7 @@ class AtomDance(object):
       rij = np.sqrt(np.sum(np.square(vij)))
       
       ux  = vkj/rkj
-      if tor is None:
+      if tor is None or tor == 0.0:
          uij = vij/rij
          rk  = np.dot(uij,ux)
          vy  = uij - rk*ux
