@@ -6,8 +6,8 @@ from ase.constraints import FixAtoms
 import matplotlib.pyplot as plt
 from math import ceil
 from .molecule import packmol
-from .dft.siesta import siesta_opt 
-from .dft.qe import qeopt # qemd,
+from .dft.siesta import siesta_md # siesta_opt 
+from .dft.qe import qemd # qeopt  
 from .dft.SinglePointEnergy import SinglePointEnergies
 from .md.gulp import nvt as gulp_nvt
 from .data.mdtodata import MDtoData
@@ -15,8 +15,8 @@ from .data.prep_data import prep_data
 from .reaxfflib import read_lib
 import tensorflow as tf
 from .md.irmd import IRMD
-from .training import train_reax
-from .training import train_mpnn
+from .trainer import train_reax
+from .trainer import train_mpnn
 from .AtomDance import AtomDance,check_zmat
 from .dingtalk import send_msg
 import numpy as np
@@ -57,15 +57,16 @@ class LearningMachine(object):
                maxiter=100,
                dt_mlmd=0.1, dt_aimd=0.3,dft_step=5,
                batch=50,max_batch=100,
-               step=20000,md_step=20,MaxMDstep=500,colFrame=10,
+               step=20000,md_step=20,MaxMDstep=200,MinMDstep=50,
+               colFrame=50,colmin=3,
                mom_step=100,
-               mpopt=1, messages=1,
-               optword='nocoul',
-               T=300,Tmax=1000,
+               mpopt=[1,1,1,1], messages=1,
+               optword='nocoul',cons=None,
+               T=300,# Tmax=1000,
                label=None,
                convergence=0.01,nconvergence=3,
-               accCriteria=0.9,lossCriteria=10.0,lossTole=1000.0,accTole=0.05,
-               accMax=0.9,accInc=1.0001,mdInc=1.2,resetAcc=1,
+               accCriteria=0.9,lossCriteria=10.0,lossTole=1.0,accTole=0.05,
+               accMax=0.9,accInc=1.0001,mdInc=1.2,resetAcc=2,
                learnWay=5,FreePairs=None,beta=None,
                FreeAtoms=None,FirstAtom=None,
                rodic=None,
@@ -75,12 +76,14 @@ class LearningMachine(object):
                nn=True,vdwnn=False,
                bo_layer=[2,1],mf_layer=[9,2],be_layer=[6,1],vdw_layer=None,#[6,1],
                be_univeral_nn='all',bo_univeral_nn='all',mf_univeral_nn='all',vdw_univeral_nn='all',
-               EnergyFunction=1,MessageFunction=1,
-               spv_bm=False,bom={'others':1.0},
-               spv_be=True,bore={'others':0.2},
-               spv_ang=True,lambda_ang=0.02,
+               BOFunction=0,EnergyFunction=1,MessageFunction=1,VdwFunction=1,
+               #spv_bm=False,bom={'others':1.0},
+               spv_be=False,beup={},belo={},
+               spv_bo=False,boc={},# bolo={},
+               spv_vdw=False,vup={},vlo={},
+               spv_ang=False,lambda_ang=0.02,
                weight={'others':2.0},
-               lambda_bd=10000.0,lambda_reg=0.0001,regularize=True,
+               lambda_bd=10000.0,lambda_reg=0.0001, # regularize=True,
                lambda_me=0.1,learning_rate=0.0001,
                writelib=5000,ffield='ffield.json',
                dft='siesta',kpts=(1,1,1),xcf='VDW',xca='DRSLL',basistype='DZP'):
@@ -102,6 +105,7 @@ class LearningMachine(object):
       self.angmax         = angmax
       self.CheckZmat      = CheckZmat
       self.convergence    = convergence
+      self.cons           = cons
       self.nconvergence   = nconvergence
       self.learnWay       = learnWay    # the way of learing
       self.ncpu           = ncpu
@@ -119,15 +123,17 @@ class LearningMachine(object):
       self.dt_mlmd        = dt_mlmd
       self.dt_aimd        = dt_aimd
       self.T              = T
-      self.Tmax           = Tmax
+      # self.Tmax         = Tmax
       self.md_step        = md_step
       self.mom_step       = mom_step
       self.beta           = beta
       self.colFrame       = colFrame
+      self.colmin         = colmin
       self.EngTole        = EngTole
       self.dEtole         = dEtole
       self.dEstop         = dEstop
       self.MaxMDstep      = MaxMDstep
+      self.MinMDstep      = MinMDstep
       self.mpopt          = mpopt
       self.optword        = optword
       self.messages       = messages
@@ -135,6 +141,7 @@ class LearningMachine(object):
       self.ffield         = ffield
       self.nn             = nn
       self.vdwnn          = vdwnn
+      self.VdwFunction    = VdwFunction
       if not self.nn:
          self.vdwnn       = False
       self.bo_layer       = bo_layer
@@ -145,18 +152,24 @@ class LearningMachine(object):
       self.be_univeral_nn = be_univeral_nn
       self.mf_univeral_nn = mf_univeral_nn
       self.vdw_univeral_nn= vdw_univeral_nn
-      self.bore           = bore
-      self.bom            = bom
-      self.spv_bm         = spv_bm
+      self.beup           = beup
+      self.belo           = belo
+      self.boc            = boc
+      #self.bolo          = bolo
+      self.vup            = vup
+      self.vlo            = vlo
+      self.spv_bo         = spv_bo
+      self.spv_vdw        = spv_vdw
       self.spv_be         = spv_be
       self.spv_ang        = spv_ang
       self.lambda_me      = lambda_me
       self.weight         = weight
       self.lambda_bd      = lambda_bd
-      self.regularize     = regularize
+      # self.regularize   = regularize
       self.lambda_reg     = lambda_reg
       self.lambda_ang     = lambda_ang
       self.learning_rate  = learning_rate
+      self.BOFunction     = BOFunction
       self.EnergyFunction = EnergyFunction
       self.MessageFunction= MessageFunction
       self.writelib       = writelib
@@ -177,11 +190,6 @@ class LearningMachine(object):
       # if self.freeatoms is None:
       #    self.freeatoms = [i for i in range(self.natom)]
 
-      if self.mpopt==1:
-         self.messages  = 1
-      elif self.mpopt==2 or self.mpopt==3:
-         self.messages  = 2
-
       if self.nn:
          self.trainer = train_mpnn
       else:
@@ -242,7 +250,7 @@ class LearningMachine(object):
       self.save_config()
       converg        = False
       nconvergence   = 0
-      unreasonable_zv,uzv_ = None,None
+      uncertainty_zv,uzv_ = None,None
       zmat_variable  = None
       zmatopt        = None
       zmats          = None
@@ -311,12 +319,12 @@ class LearningMachine(object):
           # atoms.set_constraint(self.fixatoms)
           # atoms.write('poscar.gen')                                    # for aimd
 
-          if learnWay==1: 
+          if learnWay==1 or (uncertainty_zv is not None): 
              dft_step = self.dft_step
           else:
              dft_step = int(mdsteps/self.colFrame)+1
 
-          e_aimd,eml_,dEmax_,d2Emax_,LabelDataLog = self.aimd(atoms,cwd,run_dir,dft_step,gen,learnWay)
+          e_aimd,eml_,dEmax_,d2Emax_,LabelDataLog = self.aimd(atoms,cwd,run_dir,dft_step,learnWay)
           e_siesta.append(e_aimd[0])                                   # get siesta results              
                                                                        # start training  
           trajs_ = prep_data(label=self.label,direcs=data_dir,
@@ -339,24 +347,30 @@ class LearningMachine(object):
                                                      step=self.step,writelib=self.writelib,
                                                      batch=self.batch,
                                                      convergence=self.accCriteria,
+                                                     cons=self.cons,
                                                      lossConvergence=self.lossCriteria,
                                                      nn=self.nn,vdwnn=self.vdwnn,
-                                                     mpopt=self.mpopt,optword=self.optword,
+                                                     mpopt=self.mpopt,messages=self.messages,
+                                                     optword=self.optword,
                                                      bo_layer=self.bo_layer,
                                                      mf_layer=self.mf_layer,
                                                      be_layer=self.be_layer,
                                                      vdw_layer=self.vdw_layer,
+                                                     VdwFunction=self.VdwFunction,
                                                      bo_univeral_nn=self.bo_univeral_nn,
                                                      be_univeral_nn=self.be_univeral_nn,
                                                      mf_univeral_nn=self.mf_univeral_nn,
                                                      vdw_univeral_nn=self.vdw_univeral_nn,
+                                                     BOFunction=self.BOFunction,
                                                      EnergyFunction=self.EnergyFunction,
                                                      MessageFunction=self.MessageFunction,
-                                                     bore=self.bore, bom=self.bom,
-                                                     spv_be=self.spv_be,spv_bm=self.spv_bm,
+                                                     # bore=self.bore, # bom=self.bom,
+                                                     spv_be=self.spv_be,belo=self.belo,beup=self.beup,
+                                                     spv_bo=self.spv_bo,boc=self.boc,#boup=self.boup,
+                                                     spv_vdw=self.spv_vdw,vlo=self.vlo,vup=self.vup,
                                                      spv_ang=self.spv_ang,lambda_ang=self.lambda_ang,
                                                      lambda_me=self.lambda_me,weight=self.weight,
-                                                     regularize=self.regularize,lambda_reg=self.lambda_reg,
+                                                     lambda_reg=self.lambda_reg,
                                                      lambda_bd=self.lambda_bd,
                                                      learning_rate=self.learning_rate)
                 self.load_config()
@@ -379,12 +393,13 @@ class LearningMachine(object):
              if not zmatopt is None:
                 optlog = self.a.get_optimal_zv(atoms,zmatopt,self.initConfig)
              if mdsteps>10:
-                unreasonable_zv,u_zvlo,u_zvhi = self.a.get_zmat_info(zmats)
+                uncertainty_zv,u_zvlo,u_zvhi = self.a.get_zmat_info(zmats)
 
           if mdsteps<10: mdsteps = 10
           zmats   = None
           images  = None
           zmatopt = zmat_variable
+
           if learnWay==6:
              if ms<len(self.freeatoms):
                 i = self.freeatoms[ms]
@@ -395,8 +410,8 @@ class LearningMachine(object):
              if Deformed>=1.0:
                 images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=zmat_variable,nbin=mdsteps,
                                                     zvlo=zvlo,zvhi=zvhi,traj='md.traj')
-             elif not unreasonable_zv is None:
-                images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=unreasonable_zv,nbin=mdsteps,
+             elif not uncertainty_zv is None:
+                images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=uncertainty_zv,nbin=mdsteps,
                                                     zvlo=u_zvlo,zvhi=u_zvhi,traj='md.traj')
              else:
                 e_gmd,mdsteps,Deformed,zmats,zmat_variable,zvlo,zvhi = self.mlmd(cwd,run_dir,atoms, 10000,
@@ -418,12 +433,12 @@ class LearningMachine(object):
              if self.accCriteria<self.accMax:                           # accracy adjusted dynamicly
                 self.accCriteria = min(self.accCriteria*self.accInc,self.accMax)  
           elif accu < self.accCriteria:                                 # adjust MD steps according 
-             if self.md_step>30:                                        # training results
+             if self.md_step>self.MinMDstep:                                        # training results
                 self.md_step  = int(self.md_step/self.mdInc)            #                 
 
-          if self.md_step>=self.MaxMDstep*0.5:
-             if self.T<self.Tmax:
-                self.T = self.T*self.mdInc
+          # if self.md_step>=self.MaxMDstep*0.5:
+          #    if self.T<self.Tmax:
+          #       self.T = self.T*self.mdInc
 
           self.save_config()
 
@@ -435,7 +450,7 @@ class LearningMachine(object):
           plot_energies(it,e_siesta,e_gulp)                             # plot learning status                                   
           self.learninglog(iter_,loss,accu,mdsteps,e_gulp[-1],e_siesta[-1],dEmax_,d2Emax_,LabelDataLog,
           	               learnWay,Deformed,
-                           self.a.InitZmat,unreasonable_zv,u_zvlo,u_zvhi,optlog,relaxlog)
+                           self.a.InitZmat,uncertainty_zv,u_zvlo,u_zvhi,optlog,relaxlog)
           converg_ = converg
           if diff<=self.convergence: 
              converg = True
@@ -500,7 +515,7 @@ class LearningMachine(object):
             l.write(optlog)
          l.write(' \n')
 
-  def aimd(self,atoms,cwd,run_dir,tstep,gen,learnWay):
+  def aimd(self,atoms,cwd,run_dir,tstep,learnWay):
       mkdir(run_dir)
       chdir(cwd+'/'+run_dir)
       
@@ -516,20 +531,20 @@ class LearningMachine(object):
 
       if learnWay==1:
          if self.dft=='siesta':
-            # images = siesta_md(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,us='F',tstep=tstep,
-            #                    gen=cwd+'/'+gen,FreeAtoms=self.freeatoms,
-            #                    xcf=self.xcf,xca=self.xca,basistype=self.basistype)
-            images = siesta_opt(atoms=atoms,label=self.label,ncpu=self.ncpu,VariableCell=False,us='F',
-                                tstep=tstep,gen=cwd+'/'+gen,
-                                xcf=self.xcf,xca=self.xca,basistype=self.basistype)
+            images = siesta_md(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,us='F',tstep=tstep,
+                               atoms=atoms,FreeAtoms=self.freeatoms,
+                               xcf=self.xcf,xca=self.xca,basistype=self.basistype)
+            # images = siesta_opt(atoms=atoms,label=self.label,ncpu=self.ncpu,VariableCell=False,us='F',
+            #                     tstep=tstep,gen=cwd+'/'+gen,
+            #                     xcf=self.xcf,xca=self.xca,basistype=self.basistype)
             ind_   = 'Molecular Dynamics Simulation'
          elif self.dft=='qe':
-            # images = qemd(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,tstep=tstep,
-            #               gen=cwd+'/'+gen,
-            #               kpts=self.kpts)         ## geomentry optimize
-            images = qeopt(atoms=atoms,label=self.label,ncpu=self.ncpu,
-                           gen=cwd+'/'+gen,
-                           kpts=self.kpts)         ## geomentry optimize
+            images = qemd(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,tstep=tstep,
+                          atoms=atoms,
+                          kpts=self.kpts)         ## geomentry optimize
+            # images = qeopt(atoms=atoms,label=self.label,ncpu=self.ncpu,
+            #                gen=cwd+'/'+gen,
+            #                kpts=self.kpts)         ## geomentry optimize
             ind_   = 'Geomentry Optimization'
          else:
             raise RuntimeError('-  This method not implimented!')
@@ -541,7 +556,7 @@ class LearningMachine(object):
          # print(' * files in this dir \n',listdir())
          E,E_,dEmax,d2Emax,ind_ = SinglePointEnergies('md.traj',label=self.label,EngTole=self.EngTole,
                                                  frame=tstep,select=True,
-                                                 dE=self.dEtole, # d2E=self.d2Etole,
+                                                 dE=self.dEtole, colmin=self.colmin,# d2E=self.d2Etole,
                                                  dft=self.dft,kpts=self.kpts,
                                                  xcf=self.xcf,xca=self.xca,basistype=self.basistype,
                                                  cpu=self.ncpu)
@@ -634,40 +649,42 @@ class LearningMachine(object):
       with open('options.json','w') as fj:
            options = {'accCriteria':self.accCriteria,
                     'lossCriteria':self.lossCriteria,
-                    'lossTole':self.lossTole,
-                    'rmax':self.rmax,
-                    'angmax':self.angmax,
-                    'accTole':self.accTole,
-                    'accMax':self.accMax,
+                    # 'lossTole':self.lossTole,
+                    # 'rmax':self.rmax,
+                    # 'angmax':self.angmax,
+                    # 'accTole':self.accTole,
+                    # 'accMax':self.accMax,
                     # 'accInc':self.accInc,
                     # 'resetAcc':self.resetAcc,
                     'step':self.step,
                     'md_step':self.md_step,
-                    'mom_step':self.mom_step,
+                    # 'mom_step':self.mom_step,
                     'MaxMDstep':self.MaxMDstep,
+                    'MinMDstep':self.MinMDstep,
                     'EngTole':self.EngTole,
                     'dEtole':self.dEtole,
                     'dEstop':self.dEstop,
                     'beta':self.beta,
-                    # 'mdInc':self.mdInc,
-                    'regularize':self.regularize,
+                    # 'mpopt':self.mpopt,
+                    # 'regularize':self.regularize,
                     'lambda_reg':self.lambda_reg,
                     'lambda_bd':self.lambda_bd,
                     'lambda_me':self.lambda_me,
                     'lambda_ang':self.lambda_ang,
                     'learning_rate':self.learning_rate,
-                    'rmin':self.rmin,
+                    # 'rmin':self.rmin,
                     'T':self.T,
                     'maxiter':self.maxiter,
-                    'max_batch':self.max_batch,
-                    'optword':self.optword,
+                    # 'max_batch':self.max_batch,
+                    #'optword':self.optword,
                     'learnWay':self.learnWay,
                     'colFrame':self.colFrame,
                     'CheckZmat':self.CheckZmat,
-                    'freeatoms':self.freeatoms,
-                    'freepairs':self.freepairs,
-                    'bore':self.bore,
-                    'writelib':self.writelib}
+                    'freeatoms':self.freeatoms}
+                    # 'freepairs':self.freepairs
+                    # 'bore':self.bore,
+                    # 'writelib':self.writelib
+
            if self.learnWay==1:
               options['dft_step'] = self.dft_step
            js.dump(options,fj,sort_keys=True,indent=2) 
@@ -677,41 +694,45 @@ class LearningMachine(object):
            options = js.load(fj)
            self.accCriteria  = options['accCriteria']
            self.lossCriteria = options['lossCriteria']
-           self.lossTole     = options['lossTole']
-           self.rmax         = options['rmax']
-           self.angmax       = options['angmax']
-           self.accTole      = options['accTole']
-           self.accMax     = options['accMax']
+           # self.lossTole     = options['lossTole']
+           # self.rmax         = options['rmax']
+           # self.angmax       = options['angmax']
+           # self.accTole      = options['accTole']
+           # self.accMax     = options['accMax']
            # self.accInc     = options['accInc']
            # self.resetAcc   = options['resetAcc']
            self.step         = options['step'] 
            if self.learnWay==1:
               self.dft_step     = options['dft_step'] 
            self.md_step      = options['md_step'] 
-           self.mom_step     = options['mom_step'] 
+           
+           if 'mom_step' in options: self.mom_step  = options['mom_step'] 
+           if 'colmin'   in options: self.colmin = options['colmin'] 
+
            self.MaxMDstep    = options['MaxMDstep'] 
+           self.MinMDstep    = options['MinMDstep'] 
            self.EngTole      = options['EngTole'] 
            self.dEtole       = options['dEtole'] 
            self.dEstop       = options['dEstop'] 
            self.beta         = options['beta'] 
-           # self.mdInc      = options['mdInc'] 
-           self.regularize   = options['regularize'] 
+           # self.mpopt        = options['mpopt'] 
+           # self.regularize   = options['regularize'] 
            self.lambda_reg   = options['lambda_reg'] 
            self.lambda_bd    = options['lambda_bd']
            self.lambda_me    = options['lambda_me']
            self.lambda_ang   = options['lambda_ang']
-           self.rmin         = options['rmin'] 
+           # self.rmin       = options['rmin'] 
            self.T            = options['T'] 
            self.maxiter      = options['maxiter']
-           self.max_batch    = options['max_batch']
-           self.optword      = options['optword']
+           # self.max_batch  = options['max_batch']
+           #self.optword      = options['optword']
            self.learnWay     = options['learnWay']
            self.learning_rate= options['learning_rate']
            self.colFrame     = options['colFrame']
            self.CheckZmat    = options['CheckZmat']
-           self.bore         = options['bore']
-           self.writelib     = options['writelib']
-           self.freepairs    = options['freepairs']
+           # self.bore       = options['bore']
+           # self.writelib   = options['writelib']
+           # self.freepairs    = options['freepairs']
            if self.freeatoms!= options['freeatoms']:
               self.freeatoms = options['freeatoms']
               self.a.get_freebond(self.freeatoms)
@@ -732,9 +753,9 @@ if __name__ == '__main__':
                      step=50000,md_step=100,MaxMDstep=1000,mom_step=50,colFrame=100,
                      T=500,
                      angmax=20.0,
-                     lossCriteria=20.0,lossTole=30.0,
+                     lossCriteria=20.0,lossTole=1.0,
                      CheckZmat=False,
-                     regularize=True,
+                     # regularize=True,
                      lambda_reg=0.001,lambda_bd=1000.0,lambda_me=0.001,lambda_ang=0.01,
                      learnWay=1,dft_step=50,
                      beta=0.77,# FirstAtom=4,FreeAtoms=[4,5,8], 
