@@ -144,6 +144,8 @@ class MDtoData(object):
          self.get_cpmd_frame()
       elif self.dft=='siesta':
          self.get_siesta_energy()
+      elif self.dft=='qe':
+         self.get_qe_energy()
       elif self.dft=='ase':
          if atoms==None:
             images  = self.get_ase_energy(direc)
@@ -516,6 +518,28 @@ class MDtoData(object):
          self.nframe = 1
 
 
+  def get_qe_energy(self,label='pw'):
+      if isfile(label+'.MDE'):
+         fe = open(label+'.MDE','r')
+         lines = fe.readlines()
+         fe.close()
+         l1= lines[1].split()
+         l = lines[-1].split()
+         if len(l)==0:
+            l = lines[-2].split()
+         self.nframe = int(l[0]) - int(l1[0]) + 1
+
+         self.energy_nw = [] # np.zeros([self.nframe],dtype=np.float32)
+         for line in lines:
+             l = line.split()
+             if l[1] != 'Step':
+                # f = int(l[0]) - 1
+                self.energy_nw.append(float(l[2])) # unit eV
+      else:
+         self.energy_nw = [0.0]
+         self.nframe = 1
+
+
   def get_siesta_cart(self,label='siesta'):
       fin = open('in.fdf','r') 
       lines= fin.readlines()
