@@ -10,7 +10,7 @@ import numpy as np
 
 def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
                         EngTole=0.0000001,frame=50,cpu=4,dft='siesta',kpts=(1,1,1),
-                        dE=0.2,d2E=0.1,select=False):
+                        dE=0.2,d2E=0.1,colmin=2,select=False):
     ''' get single point energy and labeling data '''
     images = Trajectory(traj)
     tframe = len(images)
@@ -19,9 +19,9 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
        if frame>1:
           ind_  = list(np.linspace(0,tframe-1,num=frame,dtype=np.int32))
        else:
-       	  ind_  = [tframe-1]
+          ind_  = [tframe-1]
     else:
-       ind_  = [i for i in range(tframe)]   
+       ind_  = [i for i in range(tframe)]
 
     if len(ind_)>1 and 0 in ind_:
        ind_.pop(0)
@@ -53,7 +53,13 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
            if select:
               if dE_>dE or extreme_point:
                  if i not in ind_:
-                    if i-1 not in ind_:
+                    ajacent_not_in = True
+                    for j in range(1,colmin):
+                        ii = i-j
+                        # ii_= i+j
+                        if ii>0 and (ii not in ind_): #and (ii_ not in ind_):
+                           ajacent_not_in = False
+                    if ajacent_not_in:
                        ind_.append(i)
 
         dEs.append(dE_)
@@ -62,13 +68,15 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
 
     ide  = np.argmax(dEs)
     id2e = np.argmax(d2Es)
+
     if (ide not in ind_) and (ide+1 not in ind_) and (ide-1 not in ind_): 
        ind_.append(ide)
     if id2e not in ind_ and (id2e+1 not in ind_) and (id2e-1 not in ind_): 
        ind_.append(id2e)
+
     ind_.sort()
 
-    LabelDataLog = '         AtomicConfigure   E(ML)  E(DFT)   Diff    dE   d2E  \n'
+    LabelDataLog = '         AtomicConfigure   E(ML)  E(DFT)   Diff    dE   d2E   \n'
     LabelDataLog+= '      --------------------------------------------------------\n'
 
     for i in ind_:
@@ -90,11 +98,11 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
         diff_ = abs(e-e_)
         LabelDataLog += '     {:3d}   {:9.5f}  {:9.5f}  {:6.6f}  {:5.4f}   {:5.4f}\n'.format(i,
                          e_,e,diff_,dE_,d2E_)
-        with open('SinglePointEnergies.log','a') as fs:  
+        with open('SinglePointEnergies.log','a') as fs:
              fs.write('%d MLP: %9.5f DFT: %9.5f Diff: %6.6f dE: %5.4f d2E: %5.4f\n' %(i,
                       e_,e,diff_,dE_,d2E_))
-            
-        if diff_>EngTole:  # or i==ind_[-1]
+
+        if diff_>EngTole:            #  or i==ind_[-1]
            his.write(atoms=atoms_)
 
     his.close()
