@@ -1,16 +1,20 @@
-from __future__ import print_function
-from .reaxfflib import read_lib,write_lib
+import json as js
+import numpy as np
 from ase import Atoms
 from ase.io import read,write
-from ase.units import GPa
-import numpy as np
+from ase.calculators.calculator import Calculator, all_changes
 from .qeq import qeq
 from .RadiusCutOff import setRcut
-import json as js
+from .reaxfflib import read_lib,write_lib
+from .neighbors import get_neighbors,get_pangle,get_ptorsion,get_phb
 from torch.autograd import Variable
 import torch
-from ase.calculators.calculator import Calculator, all_changes
-from .neighbors import get_neighbors,get_pangle,get_ptorsion,get_phb
+
+
+try:
+   from .neighbor import get_neighbors,get_pangle,get_ptorsion,get_phb
+except ImportError:
+   from .neighbors import get_neighbors,get_pangle,get_ptorsion,get_phb
 
 
 def rtaper(r,rmin=0.001,rmax=0.002):
@@ -113,8 +117,10 @@ class IRFF(Calculator):
          m                   = j['m']
          self.MolEnergy_     = j['MolEnergy']
          self.messages       = j['messages']
+         self.BOFunction     = j['BOFunction']
          self.EnergyFunction = j['EnergyFunction']
          self.MessageFunction= j['MessageFunction']
+         self.VdwFunction    = j['VdwFunction']
          self.bo_layer       = j['bo_layer']
          self.mf_layer       = j['mf_layer']
          self.be_layer       = j['be_layer']
@@ -135,7 +141,6 @@ class IRFF(Calculator):
                self.emol = self.MolEnergy_[mol_]
             else:
                self.emol = 0.0
-            
       else:
          self.p,zpe_,self.spec,self.bonds,self.offd,self.angs,self.torp,self.Hbs= \
                        read_lib(libfile=libfile,zpe=False)
@@ -145,7 +150,11 @@ class IRFF(Calculator):
          rcut          = None
          rcuta         = None
          self.vdwnn    = False
-
+         self.EnergyFunction  = 0
+         self.MessageFunction = 0
+         self.VdwFunction     = 0
+         self.p['acut']   = 0.0001
+         self.p['hbtol']  = 0.0001
       if m is None:
          self.nn=False
          
@@ -166,14 +175,12 @@ class IRFF(Calculator):
       self.set_p(m,self.bo_layer)
       self.Qe= qeq(p=self.p,atoms=self.atoms)
 
-
   def get_charge(self,cell,positions):
       self.Qe.calc(cell,positions)
       self.q   = self.Qe.q[:-1]
       qij      = np.expand_dims(self.q,axis=0)*np.expand_dims(self.q,axis=1)
       self.qij = torch.tensor(qij*14.39975840)
 
-
   def get_neighbor(self,cell,rcell,positions):
       xi    = np.expand_dims(positions,axis=0)
       xj    = np.expand_dims(positions,axis=1)
@@ -223,7 +230,6 @@ class IRFF(Calculator):
       for key in P_:
           self.P[key] = torch.from_numpy(P_[key])
 
-
   def set_rcut(self,rcut,rcuta): 
       rcut_,rcuta_,re_ = setRcut(self.bonds,rcut,rcuta,None)
       self.rcut  = rcut_    ## bond order compute cutoff
@@ -240,7 +246,6 @@ class IRFF(Calculator):
                  # self.r_cuta[i][j] = self.rcuta[bd] 
                  self.re[i][j]     = re_[bd] 
 
-
   def get_rcbo(self):
       ''' get cut-offs for individual bond '''
       self.rc_bo = {}
@@ -252,11 +257,12 @@ class IRFF(Calculator):
           rr = log_/self.p['bo1_'+bd] 
           self.rc_bo[bd]=self.p['rosi_'+ofd]*np.power(log_/self.p['bo1_'+bd],1.0/self.p['bo2_'+bd])
   
-
   def get_bondorder_uc(self):
-      self.frc = torch.where(torch.logical_and(self.r<self.rcbo_tensor,self.r>0.0001),  
-                               torch.full_like(self.r,1.0),torch.full_like(self.r,0.0))
-
+      if self.nn:
+         self.frc = 1.0
+      else:
+         self.frc = torch.where(torch.logical_and(self.r<self.rcbo_tensor,self.r>0.0001),
+                    1.0,0.0)
       self.bodiv1 = torch.div(self.r,self.P['rosi'])
       self.bopow1 = torch.pow(self.bodiv1,self.P['bo2'])
       self.eterm1 = (1.0+self.botol)*torch.exp(torch.mul(self.P['bo1'],self.bopow1))*self.frc # consist with GULP
@@ -270,20 +276,49 @@ class IRFF(Calculator):
       self.eterm3 = torch.exp(torch.mul(self.P['bo5'],self.bopow3))*self.frc
 
       if self.nn:
-         fsi_        = self.f_nn('fsi',[self.eterm1],layer=self.bo_layer[1])
-         fpi_        = self.f_nn('fpi',[self.eterm2],layer=self.bo_layer[1])
-         fpp_        = self.f_nn('fpp',[self.eterm3],layer=self.bo_layer[1])
-         
-         self.bop_si = fsi_ #*self.frc #*self.eterm1
-         self.bop_pi = fpi_ #*self.frc #*self.eterm2
-         self.bop_pp = fpp_ #*self.frc #*self.eterm3
+         if self.BOFunction==0:
+            fsi_  = self.eterm1
+            fpi_  = self.eterm2
+            fpp_  = self.eterm3
+         elif self.BOFunction==1:
+            fsi_  = self.f_nn('fsi',[self.eterm1],layer=self.bo_layer[1])
+            fpi_  = self.f_nn('fpi',[self.eterm2],layer=self.bo_layer[1])
+            fpp_  = self.f_nn('fpp',[self.eterm3],layer=self.bo_layer[1])  
+         elif self.BOFunction==2:
+            fsi_  = self.f_nn('fsi',[-self.eterm1],layer=self.bo_layer[1])
+            fpi_  = self.f_nn('fpi',[-self.eterm2],layer=self.bo_layer[1])
+            fpp_  = self.f_nn('fpp',[-self.eterm3],layer=self.bo_layer[1])
+         else:
+            raise NotImplementedError('-  BO function not supported yet!')
+         self.bop_si = fsi_*self.eye #*self.frc #*self.eterm1
+         self.bop_pi = fpi_*self.eye #*self.frc #*self.eterm2
+         self.bop_pp = fpp_*self.eye #*self.frc #*self.eterm3
       else:
          self.bop_si = taper(self.eterm1,rmin=self.botol,rmax=2.0*self.botol)*(self.eterm1-self.botol) # consist with GULP
          self.bop_pi = taper(self.eterm2,rmin=self.botol,rmax=2.0*self.botol)*self.eterm2
          self.bop_pp = taper(self.eterm3,rmin=self.botol,rmax=2.0*self.botol)*self.eterm3
 
       self.bop    = self.bop_si + self.bop_pi+self.bop_pp
+      self.Deltap = torch.sum(self.bop,1)  
 
+      if self.MessageFunction==1:
+         self.D_si = [torch.sum(self.bop_si,1)]  
+         self.D_pi = [torch.sum(self.bop_pi,1)]
+         self.D_pp = [torch.sum(self.bop_pp,1)]
+       
+      # self.N    = torch.sum(torch.where(self.bop<self.p['boc2'],0.0,1.0),1)
+      # N         = self.P['val'] - self.P['valboc'] - self.N 
+      # # S         = torch.where(N>-0.00001,-0.5,0.5)
+      # # Si        = torch.unsqueeze(S,0)
+      # # Sj        = torch.unsqueeze(S,1) 
+      # # self.S    = (Si + Sj)*self.eye
+      # Ni       = torch.unsqueeze(N,0)*self.eye
+      # Nj       = torch.unsqueeze(N,1)*self.eye
+      # cond1    = self.bop<self.p['boc2']
+      # cond2    = self.bop>=self.p['boc2']
+      # self.S   = torch.where(torch.logical_or(torch.logical_and(cond1,torch.logical_and(Ni>0,Nj>0)),
+      #                                         torch.logical_and(cond2,torch.logical_or(Ni>=0,Nj>=0))),
+      #                         -0.10,0.10)
 
   def f1(self):
       Dv  = torch.unsqueeze(self.Deltap - self.P['val'],0)    # expand_dims(self.Deltap - self.P['val'],0)
@@ -294,12 +329,10 @@ class IRFF(Calculator):
       self.f_1 = 0.5*(DIV(VAL+self.f_2,  VAL+self.f_2+self.f_3)  + 
                       DIV(VALt+self.f_2, VALt+self.f_2+self.f_3))
 
-
   def f2(self,Dv):
       self.dexpf2  = torch.exp(-self.P['boc1']*Dv)
       self.f_2     = torch.add(self.dexpf2,torch.transpose(self.dexpf2,1,0))
 
-
   def f3(self,Dv):
       self.dexpf3 = torch.exp(-self.P['boc2']*Dv)
       delta_exp   = self.dexpf3+torch.transpose(self.dexpf3,1,0)
@@ -307,7 +340,6 @@ class IRFF(Calculator):
       self.f3log  = torch.log(0.5*delta_exp )
       self.f_3    = torch.div(-1.0,self.P['boc2'])*self.f3log
 
-
   def f45(self):
       self.D_boc = self.Deltap - self.P['valboc'] # + self.p['val_'+atomi]
       
@@ -323,36 +355,39 @@ class IRFF(Calculator):
       self.f_4 = torch.div(1.0,1.0+self.f4r)
       self.f_5 = torch.div(1.0,1.0+self.f5r)
 
-
   def get_bondorder(self):
       self.f1()
       self.f45()
 
-      self.F        = self.f_1*self.f_1*self.f_4*self.f_5 
-      self.bo0      = self.bop*self.f_1*self.f_4*self.f_5   #-0.001        # consistent with GULP
-     
-      self.bo       = torch.nn.functional.relu(self.bo0 - self.atol*self.eye)      #bond-order cut-off 0.001 reaxffatol
-      self.bopi     = self.bop_pi*self.F
-      self.bopp     = self.bop_pp*self.F
-      self.bosi     = self.bo0 - self.bopi - self.bopp 
-      self.bso      = self.P['ovun1']*self.P['Desi']*self.bo0 
-      self.Delta    = torch.sum(self.bo0,1)   
+      f11    = torch.where(self.P['ovcorr']>0.0001,self.f_1,1.0)
+      f12    = torch.where(torch.logical_and(self.P['ovcorr']>0.0001,self.P['corr13']>0.0001),
+                         self.f_1,1.0)
+      F11        = f11*f12
+      F45_       = self.f_4 * self.f_5
+      F45        = torch.where(self.P['corr13']>0.0001,F45_,1.0)
+      self.F     = F11*F45
+      self.bo0   = self.bop*self.F  # -0.001          # consistent with GULP
 
+      self.bo    = torch.nn.functional.relu(self.bo0 - self.atol * self.eye)  # bond-order cut-off 0.001 reaxffatol
+      self.bopi  = self.bop_pi * self.F
+      self.bopp  = self.bop_pp * self.F
+      self.bosi  = self.bo0 - self.bopi - self.bopp
+      self.bso   = self.P['ovun1'] * self.P['Desi'] * self.bo0
+      self.Delta = torch.sum(self.bo0, 1)
 
   def f_nn(self,pre,x,layer=5):
       X   = torch.unsqueeze(torch.stack(x,dim=2),2)
 
       o   =  []
       o.append(torch.sigmoid(torch.matmul(X,self.m[pre+'wi'])+self.m[pre+'bi']))  
-                                                                    # input layer
-      for l in range(layer):                                        # hidden layer  
+                                                                       # input layer
+      for l in range(layer):                                           # hidden layer  
           o.append(torch.sigmoid(torch.matmul(o[-1],self.m[pre+'w'][l])+self.m[pre+'b'][l]))
       
       o_  = torch.sigmoid(torch.matmul(o[-1],self.m[pre+'wo']) + self.m[pre+'bo']) 
       out = torch.squeeze(o_)                                          # output layer
       return out
 
-
   def message_passing(self):
       self.H         = []    # hiden states (or embeding states)
       self.D         = []    # degree matrix
@@ -366,22 +401,73 @@ class IRFF(Calculator):
       self.D.append(self.Deltap)                # get the initial hidden state H[0]
 
       for t in range(1,self.messages+1):
-          Di_        = torch.unsqueeze(self.D[t-1],0)*self.eye
-          Dj_        = torch.unsqueeze(self.D[t-1],1)*self.eye
-        
-          Dbi        = Di_ - self.H[t-1]
-          Dbj        = Dj_ - self.H[t-1]
-
-          Fi    = self.f_nn('f'+str(t),[Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
-          Fj    = torch.transpose(Fi,0,1)
+          Di        = torch.unsqueeze(self.D[t-1],0)*self.eye
+          Dj        = torch.unsqueeze(self.D[t-1],1)*self.eye
+          # Dbi       = Di  - self.H[t-1]
+          # Dbj       = Dj  - self.H[t-1]
 
+          # V         = self.P['val']
+          # Ni        = torch.unsqueeze(V,0)*self.eye - Dbi
+          # Nj        = torch.unsqueeze(V,1)*self.eye - Dbj
+          # S       = np.where(N>=0.0,-0.125,0.125)
+          # Si        = torch.where(Ni>0.9,-0.125,0.125)
+          # Sj        = torch.where(Nj>0.9,-0.125,0.125)
+          # self.S    = (Si + Sj)*self.eye
 
+          # if self.MessageFunction==3:
+          #    Dbi = Di - self.H[t-1]
+          #    Dbj = Dj - self.H[t-1]
+          #    Fi  = self.f_nn('f'+str(t),[self.S,Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
+          #    Fj  = torch.transpose(Fi,1,0)
+          #    F   = Fi*Fj
+          #    Fsi = F[:,:,0]
+          #    Fpi = F[:,:,1]
+          #    Fpp = F[:,:,2]
+          #    self.Hsi.append(self.Hsi[t-1]*Fsi)
+          #    self.Hpi.append(self.Hpi[t-1]*Fpi)
+          #    self.Hpp.append(self.Hpp[t-1]*Fpp)
           if self.MessageFunction==1:
+             Dsi_i = torch.unsqueeze(self.D_si[t-1],0)*self.eye - self.Hsi[t-1]
+             Dsi_j = torch.unsqueeze(self.D_si[t-1],1)*self.eye - self.Hsi[t-1]
+
+             Dpi_i = torch.unsqueeze(self.D_pi[t-1],0)*self.eye - self.Hpi[t-1]
+             Dpi_j = torch.unsqueeze(self.D_pi[t-1],1)*self.eye - self.Hpi[t-1]
+
+             Dpp_i = torch.unsqueeze(self.D_pp[t-1],0)*self.eye - self.Hpp[t-1]
+             Dpp_j = torch.unsqueeze(self.D_pp[t-1],1)*self.eye - self.Hpp[t-1]
+
+             # Fi  = self.f_nn('f'+str(t),[Dsi_j,Dpi_j,Dpp_j,Dsi_i,Dpi_i,Dpp_i,self.Hsi[t-1],self.Hpi[t-1],self.Hpp[t-1]],
+             #                  layer=self.mf_layer[1])
+             Fi  = self.f_nn('f'+str(t),[Dsi_j,Dsi_i,self.Hsi[t-1],
+                                         Dpi_j,Dpi_i,self.Hpi[t-1],
+                                         Dpp_j,Dpp_i,self.Hpp[t-1]],
+                                         layer=self.mf_layer[1])
+             Fj  = torch.transpose(Fi,1,0)
              F   = Fi*Fj
-             self.Hsi.append(self.Hsi[t-1]*F)
-             self.Hpi.append(self.Hpi[t-1]*F)
-             self.Hpp.append(self.Hpp[t-1]*F)
+             Fsi = F[:,:,0]
+             Fpi = F[:,:,1]
+             Fpp = F[:,:,2]
+             self.Hsi.append(self.Hsi[t-1]*Fsi)
+             self.Hpi.append(self.Hpi[t-1]*Fpi)
+             self.Hpp.append(self.Hpp[t-1]*Fpp)
           elif self.MessageFunction==2:
+             Dbi = Di - self.H[t-1]
+             Dbj = Dj - self.H[t-1]
+             Fi  = self.f_nn('f'+str(t),[Dbj,Dbi,self.Hsi[t-1],self.Hpi[t-1],self.Hpp[t-1]],
+                             layer=self.mf_layer[1])
+             Fj  = torch.transpose(Fi,1,0)
+             F   = Fi*Fj
+             Fsi = F[:,:,0]
+             Fpi = F[:,:,1]
+             Fpp = F[:,:,2]
+             self.Hsi.append(self.Hsi[t-1]*Fsi)
+             self.Hpi.append(self.Hpi[t-1]*Fpi)
+             self.Hpp.append(self.Hpp[t-1]*Fpp)
+          elif self.MessageFunction==3:
+             Dbi   = Di  - self.H[t-1]
+             Dbj   = Dj  - self.H[t-1]
+             Fi    = self.f_nn('f'+str(t),[Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
+             Fj  = torch.transpose(Fi,1,0)
              F   = Fi*Fj
              Fsi = F[:,:,0]
              Fpi = F[:,:,1]
@@ -393,7 +479,10 @@ class IRFF(Calculator):
              raise NotImplementedError('-  Message function not supported yet!')
           self.H.append(self.Hsi[t]+self.Hpi[t]+self.Hpp[t])
           self.D.append(torch.sum(self.H[t],1) )
-
+          if self.MessageFunction==1:
+             self.D_si.append(torch.sum(self.Hsi[t],1))
+             self.D_pi.append(torch.sum(self.Hpi[t],1))
+             self.D_pp.append(torch.sum(self.Hpp[t],1))
 
   def get_bondorder_nn(self):
       self.message_passing()
@@ -401,27 +490,39 @@ class IRFF(Calculator):
       self.bopi  = self.Hpi[-1]
       self.bopp  = self.Hpp[-1]
 
-      self.bo0   = self.H[-1]
+      self.bo0   = self.H[-1] # torch.where(self.H[-1]<0.000001,0.0,self.H[-1])
       # self.fbo   = taper(self.bo0,rmin=self.botol,rmax=2.0*self.botol)
       self.bo    = torch.nn.functional.relu(self.bo0 - self.atol*self.eye)      #bond-order cut-off 0.001 reaxffatol
       self.bso   = self.P['ovun1']*self.P['Desi']*self.bo0  
       self.Delta = torch.sum(self.bo0,1)   
 
-      Di_        = torch.unsqueeze(self.Delta,0)*self.eye          # get energy layer
-      Dj_        = torch.unsqueeze(self.Delta,1)*self.eye
-      Dbi        = Di_ - self.bo0
-      Dbj        = Dj_ - self.bo0
-
+      self.Di    = torch.unsqueeze(self.Delta,0)*self.eye          # get energy layer
+      self.Dj    = torch.unsqueeze(self.Delta,1)*self.eye
+      Dbi        = self.Di - self.bo0
+      Dbj        = self.Dj - self.bo0
+       
       if self.EnergyFunction==1: 
+         self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
+      elif self.EnergyFunction==2:
+         self.esi = self.f_nn('fe',[-self.bosi,-self.bopi,-self.bopp],layer=self.be_layer[1])
+      elif self.EnergyFunction==3: 
          e_ = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
          self.esi = self.bo0*e_
-      elif self.EnergyFunction==2:
+      elif self.EnergyFunction==4:
          Fi = self.f_nn('fe',[Dbj,Dbi,self.bo0],layer=self.be_layer[1])
-         Fj = torch.transpose(Fi,0,1)
+         Fj = torch.transpose(Fi,1,0)
          self.esi  = Fi*Fj*self.bo0
-      elif self.EnergyFunction==3:
-         self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
+      elif self.EnergyFunction==5:
+         r_        = self.bodiv1
+         mors_exp1 = torch.exp(self.P['be2']*(1.0-r_))
+         mors_exp2 = torch.square(mors_exp1) 
 
+         mors_exp10= torch.exp(self.P['be2']*self.P['be1'])
+         mors_exp20= torch.square(mors_exp10) 
+         emorse    = 2.0*mors_exp1 - mors_exp2 + mors_exp20 - 2.0*mors_exp10
+         self.esi  = torch.nn.ReLU(emorse)
+      else:
+         raise NotImplementedError('-  This method is not implimented!')
 
   def get_ebond(self,cell,rcell,positions):
       self.vr    = fvr(positions)
@@ -434,7 +535,6 @@ class IRFF(Calculator):
       self.r     = torch.sqrt(torch.sum(self.vr*self.vr,2)+0.0000000001)
 
       self.get_bondorder_uc()
-      self.Deltap= torch.sum(self.bop,1)  
 
       if self.nn:
          self.get_bondorder_nn()
@@ -448,13 +548,13 @@ class IRFF(Calculator):
       self.fbo   = taper(self.bo0,rmin=self.atol,rmax=2.0*self.atol) 
       self.fhb   = taper(self.bo0,rmin=self.hbtol,rmax=2.0*self.hbtol) 
 
-      if self.EnergyFunction==1 or self.EnergyFunction==2:
+      if self.EnergyFunction>=1: # or self.EnergyFunction==2 or self.EnergyFunction==3:
          self.ebond = - self.P['Desi']*self.esi
-      elif self.EnergyFunction==3:
-         self.sieng =   self.P['Desi']*self.esi*self.bosi
-         self.pieng =   self.P['Depi']*self.esi*self.bopi
-         self.ppeng =   self.P['Depp']*self.esi*self.bopp
-         self.ebond = - self.sieng - self.pieng - self.ppeng
+      # elif self.EnergyFunction==4:
+      #    self.sieng =   self.P['Desi']*self.esi*self.bosi
+      #    self.pieng =   self.P['Depi']*self.esi*self.bopi
+      #    self.ppeng =   self.P['Depp']*self.esi*self.bopp
+      #    self.ebond = - self.sieng - self.pieng - self.ppeng
       else:
           if self.nn:
              self.sieng = torch.mul(self.P['Desi'],self.esi)
@@ -465,7 +565,8 @@ class IRFF(Calculator):
 
           self.pieng = torch.mul(self.P['Depi'],self.bopi)
           self.ppeng = torch.mul(self.P['Depp'],self.bopp)
-          self.ebond = - self.sieng - self.pieng - self.ppeng
+          self.esi   = self.sieng + self.pieng + self.ppeng
+          self.ebond = -self.esi
       self.Ebond = 0.5*torch.sum(self.ebond)
       return self.Ebond
 
@@ -755,7 +856,15 @@ class IRFF(Calculator):
             torch.div(-70.0,self.vdwcut**6.0)*torch.pow(r,6.0)+ \
             torch.div(20.0,self.vdwcut**7.0)*torch.pow(r,7.0)
       if self.vdwnn:
-         tp = self.f_nn('fv',[r],layer=self.vdw_layer[1]) # /self.P['rvdw']
+         if self.VdwFunction==1:
+            tp = self.f_nn('fv',[r],layer=self.vdw_layer[1]) # /self.P['rvdw']
+         elif self.VdwFunction==2:
+            tpi = self.f_nn('fv',[r,self.Di,self.Dj],layer=self.vdw_layer[1]) 
+            # tpj = self.f_nn('fv',[r,dj,di],layer=self.vdw_layer[1]) 
+            tpj = torch.transpose(tpi,1,0)
+            tp  = tpi*tpj
+         else:
+            raise RuntimeError('-  This method not implimented!')
       else:
          tp = tpc
       return tp,tpc
@@ -1034,7 +1143,8 @@ class IRFF(Calculator):
                      'coa1','V1','V2','V3','cot1','pen1','Devdw','Dehb']
       p_bond = ['Desi','Depi','Depp','be1','bo5','bo6','ovun1',
                 'be2','bo3','bo4','bo1','bo2',
-                'Devdw','rvdw','alfa','rosi','ropi','ropp']
+                'Devdw','rvdw','alfa','rosi','ropi','ropp',
+                'corr13','ovcorr']
       p_offd = ['Devdw','rvdw','alfa','rosi','ropi','ropp']
       self.P = {}
 
@@ -1134,10 +1244,11 @@ class IRFF(Calculator):
 
   def set_m(self,m):
       self.m = {}
-      # if self.EnergyFunction==1:
-      #    pres = ['fesi','fepi','fepp','fsi','fpi','fpp','fv']
-      # else:
-      pres = ['fe','fsi','fpi','fpp']
+      if self.BOFunction==0:
+         pres = ['fe']
+      else:
+         pres = ['fe','fsi','fpi','fpp']
+         
       if self.vdwnn:
          pres.append('fv')
 
@@ -1151,12 +1262,12 @@ class IRFF(Calculator):
               for i in range(self.natom):
                   mi_ = []
                   for j in range(self.natom):
-                      if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
-                         bd = self.atom_name[i] + '-' + self.atom_name[j]
-                         if bd not in self.bonds:
-                            bd = self.atom_name[j] + '-' + self.atom_name[i]
-                      else:
-                         bd = self.atom_name[i] 
+                      #if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
+                      bd = self.atom_name[i] + '-' + self.atom_name[j]
+                      if bd not in self.bonds:
+                         bd = self.atom_name[j] + '-' + self.atom_name[i]
+                      # else:
+                      #  bd = self.atom_name[i] 
                       key_ = key+'_'+bd
                       if key_ in m:
                          if k in ['bi','bo']:
@@ -1184,12 +1295,12 @@ class IRFF(Calculator):
                   for i in range(self.natom):
                       mi_ = []
                       for j in range(self.natom):
-                          if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
-                             bd = self.atom_name[i] + '-' + self.atom_name[j]
-                             if bd not in self.bonds:
-                                bd = self.atom_name[j] + '-' + self.atom_name[i]
-                          else:
-                             bd = self.atom_name[i] 
+                          # if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
+                          bd = self.atom_name[i] + '-' + self.atom_name[j]
+                          if bd not in self.bonds:
+                             bd = self.atom_name[j] + '-' + self.atom_name[i]
+                          # else:
+                          #   bd = self.atom_name[i] 
                           key_ = key+'_'+bd
                           if key_ in m:
                              if k == 'b':
@@ -1199,42 +1310,6 @@ class IRFF(Calculator):
                       m_.append(mi_)
                   self.m[key].append(torch.tensor(m_,dtype=torch.double))
 
-
-  def set_m_univeral(self,m,pres=['fe','fsi','fpi','fpp']):
-      # self.m = {}
-      for t in range(1,self.messages+1):
-          pres.append('f'+str(t))
-
-      for k_ in pres:
-          for k in ['wi','bi','wo','bo']:
-              key = k_+k
-              if k in ['bi','bo']:
-                 mi_ = np.expand_dims(np.expand_dims(np.expand_dims(m[key],axis=0),axis=0),axis=0)
-              else:
-                 mi_ = np.expand_dims(np.expand_dims(m[key],axis=0),axis=0)
-              self.m[key] = torch.tensor(mi_,dtype=torch.double)
-
-          for k in ['w','b']:
-              key = k_+k
-              self.m[key] = []
-              
-              if k_ in ['fesi','fepi','fepp','fe']:
-                 layer_ = self.be_layer[1]
-              elif k_ in ['fsi','fpi','fpp']:
-                 layer_ = self.bo_layer[1]
-              elif k_ =='fv':
-                 layer_ = self.vdw_layer[1]
-              else:
-                 layer_ = self.mf_layer[1]
-
-              for l in range(layer_):
-                  if k == 'b':
-                     m_ = np.expand_dims(np.expand_dims(np.expand_dims(m[key][l],axis=0),axis=0),axis=0)
-                  else:
-                     m_ = np.expand_dims(np.expand_dims(m[key][l],axis=0),axis=0)
-                  self.m[key].append(torch.tensor(m_,dtype=torch.double))
-
-
   def init_bonds(self):
       self.bonds,self.offd,self.angs,self.torp,self.Hbs = [],[],[],[],[]
       for key in self.p:
@@ -1253,7 +1328,6 @@ class IRFF(Calculator):
              self.Hbs.append(k[1])
       self.torp = self.checkTors(self.torp)
 
-
   def checkTors(self,torp):
       tors_ = torp
       for tor in tors_:
@@ -1273,7 +1347,6 @@ class IRFF(Calculator):
              torp.remove(tor3)  
       return torp 
 
-
   def check_tors(self,p_tor):
       tors = []          ### check torsion parameter
       for spi in self.spec:
@@ -1307,7 +1380,6 @@ class IRFF(Calculator):
                     self.p[key+'_'+tor] = 0.0
       return tors
       
-
   def logout(self):
       with open('irff.log','w') as fmd:
          fmd.write('\n------------------------------------------------------------------------\n')
@@ -1362,13 +1434,11 @@ class IRFF(Calculator):
          fmd.write('\n------------------------------------------------------------------------\n')
          fmd.write('\n- Machine Learning MD Completed!\n')
 
-
   def close(self):
       self.P  = None
       self.m  = None
       self.Qe = None
 
-
 if __name__ == '__main__':
    A  = read('poscar.gen')
    rf = IRFF(atoms=A,libfile='ffield')
