@@ -1,19 +1,12 @@
-from __future__ import print_function
-from .reaxfflib import read_lib,write_lib
+import json as js
+import numpy as np
 from ase import Atoms
 from ase.io import read,write
-import numpy as np
+from ase.calculators.calculator import Calculator, all_changes
 from .qeq import qeq
 from .RadiusCutOff import setRcut
-import json as js
-from ase.calculators.calculator import Calculator, all_changes
-# tf.compat.v1.enable_eager_execution()
-
-
-try:
-   from .neighbor import get_neighbors,get_pangle,get_ptorsion,get_phb
-except ImportError:
-   from .neighbors import get_neighbors,get_pangle,get_ptorsion,get_phb
+from .reaxfflib import read_lib,write_lib
+from .neighbors import get_neighbors,get_pangle,get_ptorsion,get_phb
 
 
 def rtaper(r,rmin=0.001,rmax=0.002):
@@ -66,7 +59,7 @@ def sigmoid(x):
 
 
 def relu(x):
-    return np.where(x>0.0,x,0.0)  
+    return np.maximum(0, x)
 
 
 
@@ -80,7 +73,7 @@ class IRFF_NP(object):
                nn=False,# vdwnn=False,
                messages=1,
                hbshort=6.75,hblong=7.5,
-               label="IRFF", **kwargs):
+               mol=None,label="IRFF", **kwargs):
       # Calculator.__init__(self,label=label, **kwargs)
       self.atoms        = atoms
       self.cell         = atoms.get_cell()
@@ -92,6 +85,7 @@ class IRFF_NP(object):
       self.EnergyFunction = 0
       self.messages     = messages
       self.safety_value = 0.000000001
+      self.label        = label
 
       self.p_ang  = ['theta0','val1','val2','coa1','val7','val4','pen1'] 
       self.p_hb   = ['rohb','Dehb','hb1','hb2']
@@ -104,8 +98,10 @@ class IRFF_NP(object):
          m                   = j['m']
          self.MolEnergy_     = j['MolEnergy']
          self.messages       = j['messages']
+         self.BOFunction     = j['BOFunction']
          self.EnergyFunction = j['EnergyFunction']
          self.MessageFunction= j['MessageFunction']
+         self.VdwFunction    = j['VdwFunction']
          self.bo_layer       = j['bo_layer']
          self.mf_layer       = j['mf_layer']
          self.be_layer       = j['be_layer']
@@ -118,15 +114,29 @@ class IRFF_NP(object):
          rcuta               = j['rcutBond']
          lf.close()
          self.init_bonds()
+
+         if mol is None:
+            self.emol = 0.0
+         else:
+            mol_ = mol.split('-')[0]
+            if mol_ in self.MolEnergy_:
+               self.emol = self.MolEnergy_[mol_]
+            else:
+               self.emol = 0.0
       else:
          self.p,zpe_,self.spec,self.bonds,self.offd,self.Angs,self.torp,self.Hbs= \
                        read_lib(libfile=libfile,zpe=False)
          m                = None
          self.bo_layer    = None
+         self.emol        = 0.0
          rcut             = None
          rcuta            = None
          self.vdwnn       = False
-
+         self.EnergyFunction  = 0
+         self.MessageFunction = 0
+         self.VdwFunction     = 0
+         self.p['acut']   = 0.0001
+         self.p['hbtol']  = 0.0001
       if m is None:
          self.nn=False
 
@@ -143,9 +153,10 @@ class IRFF_NP(object):
       self.hblong    = hblong
       self.set_rcut(rcut,rcuta)
       self.vdwcut    = vdwcut
-      self.d1  = np.triu(np.ones([self.natom,self.natom],dtype=np.float32),k=0)
-      self.d2  = np.triu(np.ones([self.natom,self.natom],dtype=np.float32),k=1)
-      self.eye = 1.0 - np.eye(self.natom,dtype=np.float32)
+      d              = np.ones([self.natom,self.natom],dtype=np.float32)
+      self.d1        = np.triu(d,k=0)
+      self.d2        = np.triu(d,k=1)
+      self.eye       = 1.0 - np.eye(self.natom,dtype=np.float32)
       self.check_offd()
       self.check_hb()
       self.get_rcbo()
@@ -243,12 +254,15 @@ class IRFF_NP(object):
           ofd=bd if b[0]!=b[1] else b[0]
 
           log_ = np.log((self.botol/(1.0+self.botol)))
-          rr = log_/self.p['bo1_'+bd] 
+          # rr = log_/self.p['bo1_'+bd] 
           self.rc_bo[bd]=self.p['rosi_'+ofd]*np.power(log_/self.p['bo1_'+bd],1.0/self.p['bo2_'+bd])
   
 
   def get_bondorder_uc(self):
-      self.frc = np.where(np.logical_or(self.r>self.rcbo,self.r<=0.001), 0.0,1.0)
+      if self.nn:
+         self.frc = 1.0
+      else:
+         self.frc = np.where(np.logical_or(self.r>self.rcbo,self.r<=0.0001), 0.0,1.0)
 
       self.bodiv1 = self.r/self.P['rosi']
       self.bopow1 = np.power(self.bodiv1,self.P['bo2'])
@@ -263,19 +277,35 @@ class IRFF_NP(object):
       self.eterm3 = np.exp(self.P['bo5']*self.bopow3)*self.frc
 
       if self.nn:
-         fsi_        = self.f_nn('fsi',[self.eterm1],layer=self.bo_layer[1])
-         fpi_        = self.f_nn('fpi',[self.eterm2],layer=self.bo_layer[1])
-         fpp_        = self.f_nn('fpp',[self.eterm3],layer=self.bo_layer[1])
-         
-         self.bop_si = fsi_ #*self.frc #*self.eterm1
-         self.bop_pi = fpi_ #*self.frc #*self.eterm2
-         self.bop_pp = fpp_ #*self.frc #*self.eterm3
+         if self.BOFunction==0:
+            fsi_  = self.eterm1
+            fpi_  = self.eterm2
+            fpp_  = self.eterm3
+         elif self.BOFunction==1:
+            fsi_  = self.f_nn('fsi',[self.eterm1],layer=self.bo_layer[1])
+            fpi_  = self.f_nn('fpi',[self.eterm2],layer=self.bo_layer[1])
+            fpp_  = self.f_nn('fpp',[self.eterm3],layer=self.bo_layer[1])  
+         elif self.BOFunction==2:
+            fsi_  = self.f_nn('fsi',[-self.eterm1],layer=self.bo_layer[1])
+            fpi_  = self.f_nn('fpi',[-self.eterm2],layer=self.bo_layer[1])
+            fpp_  = self.f_nn('fpp',[-self.eterm3],layer=self.bo_layer[1])
+         else:
+            raise NotImplementedError('-  BO function not supported yet!')
+         self.bop_si = fsi_*self.eye #*self.frc #*self.eterm1
+         self.bop_pi = fpi_*self.eye #*self.frc #*self.eterm2
+         self.bop_pp = fpp_*self.eye #*self.frc #*self.eterm3
       else:
          self.bop_si = taper(self.eterm1,rmin=self.botol,rmax=2.0*self.botol)*(self.eterm1-self.botol) # consist with GULP
          self.bop_pi = taper(self.eterm2,rmin=self.botol,rmax=2.0*self.botol)*self.eterm2
          self.bop_pp = taper(self.eterm3,rmin=self.botol,rmax=2.0*self.botol)*self.eterm3
       self.bop    = self.bop_si+self.bop_pi+self.bop_pp
 
+      self.Deltap= np.sum(self.bop,axis=1)  
+      if self.MessageFunction==1:
+         self.D_si = [np.sum(self.bop_si,axis=1)] 
+         self.D_pi = [np.sum(self.bop_pi,axis=1)] 
+         self.D_pp = [np.sum(self.bop_pp,axis=1)] 
+
 
   def f1(self):
       Dv  = np.expand_dims(self.Deltap - self.P['val'],axis=0)
@@ -320,17 +350,21 @@ class IRFF_NP(object):
       self.f1()
       self.f45()
 
-      self.F        = self.f_1*self.f_1*self.f_4*self.f_5 
-      self.bo0      = self.bop*self.f_1*self.f_4*self.f_5   #-0.001        # consistent with GULP
-     
-      bo_           = self.bo0 - self.atol
-      self.bo       = np.where(bo_>0.0,bo_,0.0)      #bond-order cut-off 0.001 reaxffatol
-      self.bopi     = self.bop_pi*self.F
-      self.bopp     = self.bop_pp*self.F
-      bosi_         = self.bo0 - self.bopi - self.bopp
-      self.bosi     = np.where(bosi_>0.0,bosi_,0.0)  
-      self.bso      = self.P['ovun1']*self.P['Desi']*self.bo0 
-      self.Delta    = np.sum(self.bo0,axis=1)   
+      f11    = np.where(self.P['ovcorr']>0.0001,self.f_1,1.0)
+      f12    = np.where(np.logical_and(self.P['ovcorr']>0.0001,self.P['corr13']>0.0001),
+                         self.f_1,1.0)
+      self.F11   = f11*f12
+      F45_       = self.f_4 * self.f_5
+      self.F45   = np.where(self.P['corr13']>0.0001,F45_,1.0)
+      self.F     = self.F11*self.F45
+      self.bo0   = self.bop*self.F  # -0.001        # consistent with GULP
+
+      self.bo    = relu(self.bo0 - self.atol * self.eye)  # bond-order cut-off 0.001 reaxffatol
+      self.bopi  = self.bop_pi * self.F
+      self.bopp  = self.bop_pp * self.F
+      self.bosi  = self.bo0 - self.bopi - self.bopp
+      self.bso   = self.P['ovun1'] * self.P['Desi'] * self.bo0
+      self.Delta = np.sum(self.bo0, 1)
 
 
   def f_nn(self,pre,x,layer=5):
@@ -360,30 +394,84 @@ class IRFF_NP(object):
       self.Hpp.append(self.bop_pp)              # 
       self.D.append(self.Deltap)                # get the initial hidden state H[0]
 
+
       for t in range(1,self.messages+1):
-          Di_        = np.expand_dims(self.D[t-1],axis=0)*self.eye
-          Dj_        = np.expand_dims(self.D[t-1],axis=1)*self.eye
-        
-          Dbi        = Di_ - self.H[t-1]
-          Dbj        = Dj_ - self.H[t-1]
-
-          # self.Fi  = self.f_nn('f',[Di,Dj,Dbi,Dbj],layer=self.mf_layer[1])
-          Fi    = self.f_nn('f'+str(t),[Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
+          Di        = np.expand_dims(self.D[t-1],axis=0)*self.eye
+          Dj        = np.expand_dims(self.D[t-1],axis=1)*self.eye
+          # Dbi       = Di  - self.H[t-1]
+          # Dbj       = Dj  - self.H[t-1]
 
+          # self.N  = np.sum(np.ceil(np.where(self.bop<self.botol,0.0,self.bop)),axis=1)
+          # V         = self.P['val'] #-1.0
+          # self.Ni   = np.expand_dims(V,axis=0)*self.eye - Dbi
+          # self.Nj   = np.expand_dims(V,axis=1)*self.eye - Dbj
+          # S       = np.where(N>=0.0,-0.125,0.125)
+          # Si        = np.where(self.Ni>0.9,-0.125,0.125)
+          # Sj        = np.where(self.Nj>0.9,-0.125,0.125)
+          # self.S    = (Si + Sj)*self.eye
+          
+          # if self.MessageFunction==3:
+          #    Dbi   = Di - self.H[t-1]
+          #    Dbj   = Dj - self.H[t-1]
+          #    Fi    = self.f_nn('f'+str(t),[self.S,Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
+          #    Fj    = np.transpose(Fi,[1,0,2])
+          #    F     = Fi*Fj
+          #    self.F.append(F)
+          #    Fsi   = F[:,:,0]
+          #    Fpi   = F[:,:,1]
+          #    Fpp   = F[:,:,2]
+          #    self.Hsi.append(self.Hsi[t-1]*Fsi)
+          #    self.Hpi.append(self.Hpi[t-1]*Fpi)
+          #    self.Hpp.append(self.Hpp[t-1]*Fpp)
           if self.MessageFunction==1:
-             Fj    = np.transpose(Fi,[1,0])
-             F     = Fi*Fj
-             self.F.append(F)
-             self.Hsi.append(self.Hsi[t-1]*F)
-             self.Hpi.append(self.Hpi[t-1]*F)
-             self.Hpp.append(self.Hpp[t-1]*F)
-          elif self.MessageFunction==2:
-             Fj    = np.transpose(Fi,[1,0,2])
-             F     = Fi*Fj
-             self.F.append(F)
+             Dsi_i = np.expand_dims(self.D_si[t-1],axis=0)*self.eye - self.Hsi[t-1]
+             Dsi_j = np.expand_dims(self.D_si[t-1],axis=1)*self.eye - self.Hsi[t-1]
+
+             Dpi_i = np.expand_dims(self.D_pi[t-1],axis=0)*self.eye - self.Hpi[t-1]
+             Dpi_j = np.expand_dims(self.D_pi[t-1],axis=1)*self.eye - self.Hpi[t-1]
+
+             Dpp_i = np.expand_dims(self.D_pp[t-1],axis=0)*self.eye - self.Hpp[t-1]
+             Dpp_j = np.expand_dims(self.D_pp[t-1],axis=1)*self.eye - self.Hpp[t-1]
+
+             # Fi  = self.f_nn('f'+str(t),[Dsi_j,Dpi_j,Dpp_j,Dsi_i,Dpi_i,Dpp_i,self.Hsi[t-1],self.Hpi[t-1],self.Hpp[t-1]],
+             #                  layer=self.mf_layer[1])
+             Fi  = self.f_nn('f'+str(t),[Dsi_j,Dsi_i,self.Hsi[t-1],
+                                         Dpi_j,Dpi_i,self.Hpi[t-1],
+                                         Dpp_j,Dpp_i,self.Hpp[t-1]],
+                                         layer=self.mf_layer[1])
+             Fj  = np.transpose(Fi,[1,0,2])
+             F   = Fi*Fj
              Fsi = F[:,:,0]
              Fpi = F[:,:,1]
              Fpp = F[:,:,2]
+             self.F.append(F)
+             self.Hsi.append(self.Hsi[t-1]*Fsi)
+             self.Hpi.append(self.Hpi[t-1]*Fpi)
+             self.Hpp.append(self.Hpp[t-1]*Fpp)
+          elif self.MessageFunction==2:
+             Dbi   = Di  - self.H[t-1]
+             Dbj   = Dj  - self.H[t-1]
+             Fi    = self.f_nn('f'+str(t),[Dbj,Dbi,self.Hsi[t-1],self.Hpi[t-1],self.Hpp[t-1]],
+                               layer=self.mf_layer[1])
+             Fj    = np.transpose(Fi,[1,0,2])
+             F     = Fi*Fj
+             Fsi = F[:,:,0]
+             Fpi = F[:,:,1]
+             Fpp = F[:,:,2]
+             self.F.append(F)
+             self.Hsi.append(self.Hsi[t-1]*Fsi)
+             self.Hpi.append(self.Hpi[t-1]*Fpi)
+             self.Hpp.append(self.Hpp[t-1]*Fpp)
+          elif self.MessageFunction==3:
+             Dbi   = Di  - self.H[t-1]
+             Dbj   = Dj  - self.H[t-1]
+             Fi    = self.f_nn('f'+str(t),[Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
+             Fj    = np.transpose(Fi,[1,0,2])
+             F     = Fi*Fj
+             self.F.append(F)
+             Fsi   = F[:,:,0]
+             Fpi   = F[:,:,1]
+             Fpp   = F[:,:,2]
              self.Hsi.append(self.Hsi[t-1]*Fsi)
              self.Hpi.append(self.Hpi[t-1]*Fpi)
              self.Hpp.append(self.Hpp[t-1]*Fpp)
@@ -391,7 +479,10 @@ class IRFF_NP(object):
              raise NotImplementedError('-  Message function not supported yet!')
           self.H.append(self.Hsi[t]+self.Hpi[t]+self.Hpp[t])
           self.D.append(np.sum(self.H[t],axis=1))  
-
+          if self.MessageFunction==1:
+             self.D_si.append(np.sum(self.Hsi[t],axis=1))
+             self.D_pi.append(np.sum(self.Hpi[t],axis=1))
+             self.D_pp.append(np.sum(self.Hpp[t],axis=1))
 
   def get_bondorder_nn(self):
       self.message_passing()
@@ -399,27 +490,39 @@ class IRFF_NP(object):
       self.bopi  = self.Hpi[-1]
       self.bopp  = self.Hpp[-1]
 
-      self.bo0   = self.H[-1] # self.bosi + self.bopi + self.bopp
+      self.bo0   = self.H[-1] # np.where(self.H[-1]<0.0000001,0.0,self.H[-1]) # self.bosi + self.bopi + self.bopp
+
       # self.fbo   = taper(self.bo0,rmin=self.botol,rmax=2.0*self.botol)
       self.bo    = relu(self.bo0 - self.atol*self.eye)     # bond-order cut-off 0.001 reaxffatol
       self.bso   = self.P['ovun1']*self.P['Desi']*self.bo0  
       self.Delta = np.sum(self.bo0,axis=1)   
 
-      Di_        = np.expand_dims(self.Delta,axis=0)*self.eye          # get energy layer
-      Dj_        = np.expand_dims(self.Delta,axis=1)*self.eye
-      Dbi        = Di_ - self.bo0
-      Dbj        = Dj_ - self.bo0
+      self.Di    = np.expand_dims(self.Delta,axis=0)*self.eye          # get energy layer
+      self.Dj    = np.expand_dims(self.Delta,axis=1)*self.eye
+      Dbi        = self.Di - self.bo0
+      Dbj        = self.Dj - self.bo0
 
       if self.EnergyFunction==1: 
-         e_ = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
-         self.esi = self.bo0*e_
+         self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])  
       elif self.EnergyFunction==2:
+         self.esi = self.f_nn('fe',[-self.bosi,-self.bopi,-self.bopp],layer=self.be_layer[1]) 
+      elif self.EnergyFunction==3: 
+         e_ = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])  
+         self.esi = self.bo0*e_
+      elif self.EnergyFunction==4:
          Fi = self.f_nn('fe',[Dbj,Dbi,self.bo0],layer=self.be_layer[1])
          Fj = np.transpose(Fi,[1,0])
          self.esi = Fi*Fj*self.bo0
-      elif self.EnergyFunction==3:
-         self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
-
+      elif self.EnergyFunction==5:
+         r_        = self.bodiv1
+         mors_exp1 = np.exp(self.P['be2']*(1.0-r_))
+         mors_exp2 = np.square(mors_exp1) 
+         mors_exp10= np.exp(self.P['be2']*self.P['be1'])
+         mors_exp20= np.square(mors_exp10) 
+         emorse    = 2.0*mors_exp1 - mors_exp2 + mors_exp20 - 2.0*mors_exp10
+         self.esi  = np.maximum(0,emorse)
+      else:
+         raise NotImplementedError('-  This method is not implimented!')
 
   def get_ebond(self,cell,rcell,positions):
       self.vr    = fvr(positions)
@@ -432,7 +535,6 @@ class IRFF_NP(object):
       self.r     = np.sqrt(np.sum(self.vr*self.vr,axis=2)+self.safety_value)
 
       self.get_bondorder_uc()
-      self.Deltap= np.sum(self.bop,axis=1)  
 
       if self.nn:
          self.get_bondorder_nn()
@@ -446,13 +548,13 @@ class IRFF_NP(object):
       self.fbo   = taper(self.bo0,rmin=self.atol,rmax=2.0*self.atol) 
       self.fhb   = taper(self.bo0,rmin=self.hbtol,rmax=2.0*self.hbtol) 
 
-      if self.EnergyFunction==1 or self.EnergyFunction==2:
+      if self.EnergyFunction>=1: # or self.EnergyFunction==2 or self.EnergyFunction==3:
          self.ebond = - self.P['Desi']*self.esi
-      elif self.EnergyFunction==3:
-         self.sieng =   self.P['Desi']*self.esi*self.bosi
-         self.pieng =   self.P['Depi']*self.esi*self.bopi
-         self.ppeng =   self.P['Depp']*self.esi*self.bopp
-         self.ebond = - self.sieng - self.pieng - self.ppeng
+      # elif self.EnergyFunction==4:
+      #    self.sieng =   self.P['Desi']*self.esi*self.bosi
+      #    self.pieng =   self.P['Depi']*self.esi*self.bopi
+      #    self.ppeng =   self.P['Depp']*self.esi*self.bopp
+      #    self.ebond = - self.sieng - self.pieng - self.ppeng
       else:
          if self.nn:
             self.sieng = np.multiply(self.P['Desi'],self.esi)
@@ -463,7 +565,8 @@ class IRFF_NP(object):
 
          self.pieng = np.multiply(self.P['Depi'],self.bopi)
          self.ppeng = np.multiply(self.P['Depp'],self.bopp)
-         self.ebond = - self.sieng - self.pieng - self.ppeng
+         self.esi   = self.sieng + self.pieng + self.ppeng
+         self.ebond = - self.esi
       self.Ebond = 0.5*np.sum(self.ebond)
       return self.Ebond
 
@@ -757,7 +860,15 @@ class IRFF_NP(object):
             np.divide(-70.0,np.power(self.vdwcut,6.0))*np.power(r,6.0)+ \
             np.divide(20.0,np.power(self.vdwcut,7.0))*np.power(r,7.0)
       if self.vdwnn:
-         tp = self.f_nn('fv',[r],layer=self.vdw_layer[1]) # /self.P['rvdw']
+         if self.VdwFunction==1:
+            tp = self.f_nn('fv',[r],layer=self.vdw_layer[1]) # /self.P['rvdw']
+         elif self.VdwFunction==2:
+            tpi = self.f_nn('fv',[r,self.Di,self.Dj],layer=self.vdw_layer[1]) 
+            # tpj = self.f_nn('fv',[r,self.Dj,self.Di],layer=self.vdw_layer[1]) 
+            tpj = np.transpose(tpi,[1,0])
+            tp  = tpi*tpj
+         else:
+            raise RuntimeError('-  This method not implimented!')
       else:
          tp = tpc
       return tp,tpc
@@ -767,6 +878,8 @@ class IRFF_NP(object):
       self.evdw = 0.0
       self.ecoul= 0.0
       nc = 0
+      #Di_        = np.expand_dims(self.D[t-1],axis=0)*self.eye
+      #Dj_        = np.expand_dims(self.D[t-1],axis=1)*self.eye
       for i in range(-1,2):
           for j in range(-1,2):
               for k in range(-1,2):
@@ -776,7 +889,7 @@ class IRFF_NP(object):
 
                   gm3  = np.power(np.divide(1.0,self.P['gamma']),3.0)
                   r3   = np.power(r,3.0)
-                  fv_   = np.where(np.logical_and(r>0.0000001,r<=self.vdwcut),1.0,0.0)
+                  fv_  = np.where(np.logical_and(r>0.0000001,r<=self.vdwcut),1.0,0.0)
                   
                   if nc<13:
                      fv = fv_*self.d1
@@ -887,6 +1000,7 @@ class IRFF_NP(object):
          self.messages       = j['messages']
          self.EnergyFunction = j['EnergyFunction']
          self.MessageFunction= j['MessageFunction']
+         self.VdwFunction    = j['VdwFunction']
          self.bo_layer = j['bo_layer']
          self.mf_layer = j['mf_layer']
          self.be_layer = j['be_layer']
@@ -1072,7 +1186,8 @@ class IRFF_NP(object):
                      'coa1','V1','V2','V3','cot1','pen1','Devdw','Dehb']
       p_bond = ['Desi','Depi','Depp','bo5','bo6','ovun1',
                 'be1','be2','bo3','bo4','bo1','bo2',
-                'Devdw','rvdw','alfa','rosi','ropi','ropp']
+                'Devdw','rvdw','alfa','rosi','ropi','ropp',
+                'corr13','ovcorr']
       p_offd = ['Devdw','rvdw','alfa','rosi','ropi','ropp']
       self.P = {}
        
@@ -1101,7 +1216,7 @@ class IRFF_NP(object):
           for i in range(self.natom):
                 sp = self.atom_name[i]
                 self.P[key][i] = self.p[key+'_'+sp]*unit_
-      self.zpe = -np.sum(self.P['atomic'])
+      self.zpe = -np.sum(self.P['atomic']) + self.emol
 
       for key in ['boc3','boc4','boc5','gamma','gammaw']:
           self.P[key] = np.zeros([self.natom,self.natom],dtype=np.float32)
@@ -1149,10 +1264,11 @@ class IRFF_NP(object):
 
   def set_m(self,m):
       self.m = {}
-      # if self.EnergyFunction==1:
-      #    pres = ['fesi','fepi','fepp','fsi','fpi','fpp','fv']
-      # else:
-      pres = ['fe','fsi','fpi','fpp']
+      if self.BOFunction==0:
+         pres = ['fe']
+      else:
+         pres = ['fe','fsi','fpi','fpp']
+         
       if self.vdwnn:
          pres.append('fv')
 
@@ -1166,12 +1282,12 @@ class IRFF_NP(object):
               for i in range(self.natom):
                   mi_ = []
                   for j in range(self.natom):
-                      if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
-                         bd = self.atom_name[i] + '-' + self.atom_name[j]
-                         if bd not in self.bonds:
-                            bd = self.atom_name[j] + '-' + self.atom_name[i]
-                      else:
-                         bd = self.atom_name[i] 
+                      # if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
+                      bd = self.atom_name[i] + '-' + self.atom_name[j]
+                      if bd not in self.bonds:
+                         bd = self.atom_name[j] + '-' + self.atom_name[i]
+                      # else:
+                      #   bd = self.atom_name[i] 
                       key_ = key+'_'+bd
                       if key_ in m:
                          if k in ['bi','bo']:
@@ -1199,12 +1315,12 @@ class IRFF_NP(object):
                   for i in range(self.natom):
                       mi_ = []
                       for j in range(self.natom):
-                          if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
-                             bd = self.atom_name[i] + '-' + self.atom_name[j]
-                             if bd not in self.bonds:
-                                bd = self.atom_name[j] + '-' + self.atom_name[i]
-                          else:
-                             bd = self.atom_name[i] 
+                          # if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
+                          bd = self.atom_name[i] + '-' + self.atom_name[j]
+                          if bd not in self.bonds:
+                             bd = self.atom_name[j] + '-' + self.atom_name[i]
+                          # else:
+                          #   bd = self.atom_name[i] 
                           key_ = key+'_'+bd
                           if key_ in m:
                              if k == 'b':
