@@ -1,4 +1,3 @@
-from __future__ import print_function
 from os import system
 import numpy as np
 from ase import Atoms
@@ -8,6 +7,25 @@ from ase.calculators.singlepoint import SinglePointCalculator
 import matplotlib.pyplot as plt
 from ..molecule import press_mol
 
+input_sample = '''
+md qiterative conv;
+
+temperature        350.000000 K;
+timestep           0.100000 fs;
+production         0.100000 ps;
+equilibration      0.0 ps;
+write              1;
+sample             1;
+
+title;
+GULP calculation;
+end;
+
+library reax;
+output movie xyz his.xyz;
+dump 100 restart.grs;
+
+'''
 
 def write_gulp_in(A,runword='gradient qiterative nosymmetry conv verb debu',
                   T=298.0,
@@ -330,7 +348,7 @@ def nvt(atoms=None,gen='poscar.gen',T=350,time_step=0.1,tot_step=100,
                   time_step=time_step,
                   tot_step=tot_step,
                   lib=lib)
-    # print('\n-  running gulp nvt ...')
+    print('\n-  running gulp nvt ...')
     if ncpu>1:
       system('mpirun -n %d gulp<inp-gulp>gulp.out' %ncpu)
     else:
@@ -352,3 +370,37 @@ def npt(T=350,time_step=0.1,tot_step=10.0,ncpu=1):
     xyztotraj('his.xyz')
 
 
+def gulp(atoms=None,gen='poscar.gen',inp=None,out='gulp.out',ncpu=1):
+    if atoms is None:
+       atoms = read(gen)
+
+    if inp is None:
+       inp = input_sample
+     
+    with open('gin','w') as finp:
+         inp_ = inp.split(';')
+         for line in inp_:
+             print(line,file=finp)
+
+         print('#',file=finp)
+         print('vectors',file=finp)
+         cell = atoms.get_cell()
+         for a in cell:
+             print(a[0],a[1],a[2],file=finp)
+         print('\n',file=finp)
+
+         print('cartesian',file=finp)
+         pos = atoms.get_positions()
+         symb= atoms.get_chemical_symbols()
+         for sp,x in zip(symb,pos):
+             print(sp,'core',x[0],x[1],x[2],0.0,1.0,0.0,1,1,1,file=finp)
+     
+    print('-  running gulp ...')
+    if ncpu>1:
+       system('mpirun -n %d gulp< %s > %s' %(ncpu,inp,out))
+    else:
+       system('gulp< %s > %s' %(inp,out))
+    xyztotraj('his.xyz')
+    print('-  Job completed.')
+
+
