@@ -4,7 +4,10 @@ from .emdk import get_structure
 import numpy as np
 from ase.io import read,write
 from ase import Atoms
-from .getNeighbors import get_neighbors,find_mole
+try:
+   from .getNeighbor import get_neighbors,find_mole
+except ImportError:
+   from .getNeighbors import get_neighbors,find_mole
 
 
 def get_mol(mol=None):
