@@ -1,4 +1,3 @@
-from __future__ import print_function
 import matplotlib.pyplot as plt
 from os import system, getcwd, chdir,listdir,environ,makedirs
 from os.path import isfile,exists,isdir
@@ -121,11 +120,15 @@ class ReaxFF(object):
                batch_size=200,sample='uniform',
                hbshort=6.75,hblong=7.5,
                vdwcut=10.0,
+               atol=0.001,
+               hbtol=0.001,
                bore={'others':0.0},
                weight={'others':1.0},
+               spv_vdw=False,vlo={'others':[(0.0,0.0)]},vup={'others':[(10.0,0.0)]},
                interactive=False,
                ro_scale=0.1,
                clip_op=True,
+               clip={'others':(0.4,1.0)},   # parameter range
                InitCheck=True,
                resetDeadNeuron=False,
                optmol=True,
@@ -133,6 +136,7 @@ class ReaxFF(object):
                nn=False,
                nnopt=True,
                vdwnn=False,
+               VdwFunction=0,
                bo_layer=[8,4],
                spec=[],
                sort=False,
@@ -141,6 +145,7 @@ class ReaxFF(object):
                fromPop=False,
                board=False,
                lambda_bd=100000.0,
+               fixrcbo=True,
                to_train=True,
                optMethod='ADAM',
                maxstep=60000,
@@ -174,6 +179,7 @@ class ReaxFF(object):
       self.ro_scale      = ro_scale
       self.conf_vale     = conf_vale
       self.clip_op       = clip_op
+      self.clip          = clip
       self.InitCheck     = InitCheck
       self.resetDeadNeuron = resetDeadNeuron
       self.hbshort       = hbshort
@@ -181,6 +187,10 @@ class ReaxFF(object):
       self.nn            = nn
       self.nnopt         = nnopt
       self.vdwnn         = vdwnn
+      self.VdwFunction   = VdwFunction if vdwnn else 0
+      self.spv_vdw       = spv_vdw
+      self.vlo           = vlo
+      self.vup           = vup
       self.bo_layer      = bo_layer
       self.weight        = weight
       self.spec          = spec
@@ -201,6 +211,9 @@ class ReaxFF(object):
       self.huber_d       = huber_d
       self.ncpu          = ncpu
       self.bore          = bore
+      self.atol          = atol     # angle bond-order tolerence
+      self.hbtol         = hbtol    # hydrogen-bond bond-order tolerence
+      self.fixrcbo       = fixrcbo
       self.m_,self.m     = None,None
 
       self.rcut,self.rcuta,self.re = self.read_lib()
@@ -210,6 +223,9 @@ class ReaxFF(object):
          self.ic = Init_Check(re=self.re,nanv=nanv)
          self.p_,self.m_ = self.ic.check(self.p_,self.m_,resetDeadNeuron=self.resetDeadNeuron)
 
+      self.p_['acut']    = self.atol        
+      self.p_['hbtol']   = self.hbtol    
+
       self.set_neurons()
       self.lambda_bd     = lambda_bd
       self.logger        = logger('training.log')
@@ -352,6 +368,8 @@ class ReaxFF(object):
       self.nhb      = self.lk.nhb
       self.hblink   = self.lk.hblink
       self.hij      = self.lk.hij
+      self.vi       = self.lk.vi
+      self.vj       = self.lk.vj
 
 
   def memory(self):
@@ -375,7 +393,7 @@ class ReaxFF(object):
       self.sieng,self.pieng,self.ppeng = {},{},{}
 
       self.Delta_e,self.DE,self.nlp,self.Delta_lp,self.Dlp = {},{},{},{},{}
-      self.Dpi,self.BSO,self.BOPI,self.Delta_lpcorr = {},{},{},{}
+      self.Dpi,self.BSO,self.BOPI,self.Delta_lpcorr,self.NLP = {},{},{},{},{}
       self.explp,self.EL,self.elone,self.Elone,self.ELONE = {},{},{},{},{}
       self.EOV,self.Eover,self.eover,self.otrm1,self.otrm2 =  {},{},{},{},{}
 
@@ -501,7 +519,15 @@ class ReaxFF(object):
              self.loss[mol] = tf.compat.v1.losses.mean_squared_error(self.dft_energy[mol],self.E[mol])
           elif self.losFunc == 'huber':
              self.loss[mol] = tf.compat.v1.losses.huber_loss(self.dft_energy[mol],self.E[mol],delta=self.huber_d)
-
+          elif self.losFunc == 'CrossEntropy':
+             y_min = tf.reduce_min(self.dft_energy[mol])
+             a_min = tf.reduce_min(self.E[mol])
+             norm  = tf.minimum(y_min,a_min) - 0.00000001
+             y     = self.dft_energy[mol]/norm
+             y_    = self.E[mol]/norm
+             self.loss[mol] =  (-1.0/self.batch)*tf.reduce_sum(y*tf.math.log(y_)+(1-y)*tf.math.log(1.0-y_))
+          else:
+             raise NotImplementedError('-  This function not supported yet!')
           sum_edft = tf.reduce_sum(input_tensor=tf.abs(self.dft_energy[mol]-self.zpe[mol])) # self.max_e[mol]
           self.accur[mol] = 1.0 - tf.reduce_sum(input_tensor=tf.abs(self.E[mol]-self.dft_energy[mol]))/(sum_edft+0.00000001)
           
@@ -519,55 +545,28 @@ class ReaxFF(object):
           mols = mol.split('-')[0] 
           self.ME += tf.square(self.MolEnergy[mols])
 
-      self.loss_atol = self.supervise()
-      self.Loss     += self.loss_atol
+      self.loss_penalty = self.supervise()
+      self.Loss     += self.loss_penalty
       
       if self.optmol:
          self.Loss  += self.ME*self.lambda_me
       self.accuracy  = self.accuracy/self.nmol
 
 
-  def supervise(self):
-      ''' supervised learning term'''
-      l_atol = 0.0
-      self.diffa,self.diffb,self.diffe = {},{},{}
-      for bd in self.bonds: 
-          [atomi,atomj] = bd.split('-') 
-          if self.nbd[bd]>0:
-             if bd in self.bore:
-                bore_ = self.bore[bd]
-             else:
-                bore_ = self.bore['others']
-             fao = tf.where(tf.greater(self.rbd[bd],self.rcuta[bd]),
-                            tf.ones_like(self.rbd[bd]),tf.zeros_like(self.rbd[bd]))
-             self.diffa[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu(self.bo0[bd]*fao-self.atol))
-             l_atol = tf.add(self.diffa[bd],l_atol)
-
-             fbo = tf.where(tf.greater(self.rbd[bd],self.rcut[bd]),
-                            tf.ones_like(self.rbd[bd]),tf.zeros_like(self.rbd[bd]))
-             self.diffb[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu(self.bop[bd]*fbo-self.botol))
-             l_atol = tf.add(self.diffb[bd],l_atol)
-
-             fe  = tf.where(tf.less_equal(self.rbd[bd],self.re[bd]),
-                            tf.ones_like(self.rbd[bd]),tf.zeros_like(self.rbd[bd]))
-             self.diffe[bd] = tf.reduce_sum(input_tensor=tf.nn.relu((bore_-self.bo0[bd]))*fe)
-             
-             l_atol = tf.add(self.diffe[bd],l_atol)
-             l_atol = tf.add(tf.nn.relu(self.rc_bo[bd]-self.rcut[bd]),l_atol)
-      return l_atol*self.lambda_bd
-
-
-  def get_bond_energy(self):
+  def get_delta(self):
+      ''' compute the uncorrected Delta: the sum of BO '''
       self.BOP = tf.zeros([1,self.batch])   # for ghost atom, the value is zero
       for bd in self.bonds:
           if self.nbd[bd]>0:
-             with tf.compat.v1.name_scope('BOuc_'+bd):
-                self.get_bondorder_uc(bd)
-                self.BOP = tf.concat([self.BOP,self.bop[bd]],0)
+             self.get_bondorder_uc(bd)
+             self.BOP = tf.concat([self.BOP,self.bop[bd]],0)
   
       self.Dp     = tf.gather_nd(self.BOP,self.dlist)  
       self.Deltap = tf.reduce_sum(input_tensor=self.Dp,axis=1,name='Deltap')
 
+  def get_bond_energy(self):
+      self.get_delta()
+
       self.BO0    = tf.zeros([1,self.batch])   # for ghost atom, the value is zero
       self.BO     = tf.zeros([1,self.batch])
       self.BOPI   = tf.zeros([1,self.batch])
@@ -632,7 +631,7 @@ class ReaxFF(object):
           if self.nsp[sp]==0:
              continue
           self.eatom[sp] = -tf.ones([self.nsp[sp]])*self.p['atomic_'+sp]
-          self.delta[sp]     = tf.gather_nd(self.Delta,self.atomlist[sp])
+          self.delta[sp] = tf.gather_nd(self.Delta,self.atomlist[sp])
           self.dang[sp]  = self.delta[sp] - self.p['valang_'+sp]
 
           self.get_elone(sp,self.delta[sp]) 
@@ -671,19 +670,19 @@ class ReaxFF(object):
       for mol in self.mols:
           mols = mol.split('-')[0] 
           if self.optword.find('noover')>=0:
-          	 self.elone[mol] = 0.0
+             self.elone[mol] = 0.0
           else:
              self.Elone[mol] = tf.gather_nd(self.ELONE,self.atomlink[mol])  
              self.elone[mol] = tf.reduce_sum(input_tensor=self.Elone[mol],axis=0,name='lonepairenergy')
 
           if self.optword.find('noover')>=0:
-          	 self.eover[mol] = 0.0
+             self.eover[mol] = 0.0
           else:
              self.Eover[mol] = tf.gather_nd(self.EOVER,self.atomlink[mol])  
              self.eover[mol] = tf.reduce_sum(input_tensor=self.Eover[mol],axis=0,name='overenergy')
           
           if self.optword.find('noover')>=0:
-          	 self.eunder[mol] = 0.0
+             self.eunder[mol] = 0.0
           else:
              self.Eunder[mol] = tf.gather_nd(self.EUNDER,self.atomlink[mol])  
              self.eunder[mol] = tf.reduce_sum(input_tensor=self.Eunder[mol],axis=0,name='underenergy')
@@ -693,21 +692,20 @@ class ReaxFF(object):
 
 
   def get_elone(self,atom,D):
-      NLPOPT             = 0.5*(self.p['vale_'+atom] - self.p['val_'+atom])
+      self.NLP[atom]     = 0.5*(self.p['vale_'+atom] - self.p['val_'+atom])
       self.Delta_e[atom] = 0.5*(D - self.p['vale_'+atom])
       self.DE[atom]      = -tf.nn.relu(-tf.math.ceil(self.Delta_e[atom])) 
       self.nlp[atom]     = -self.DE[atom] + tf.exp(-self.p['lp1']*4.0*tf.square(1.0+self.Delta_e[atom]-self.DE[atom]))
       
-      self.Delta_lp[atom] = NLPOPT-self.nlp[atom]                                        # nan error
+      self.Delta_lp[atom]= self.NLP[atom]-self.nlp[atom]                                        # nan error
       # Delta_lp  = tf.clip_by_value(self.Delta_lp[atom],-1.0,10.0)  # temporary solution
       Delta_lp  = tf.nn.relu(self.Delta_lp[atom]+1) -1
 
-      self.explp[atom]    = 1.0+tf.exp(-self.p['lp3']*Delta_lp)
+      self.explp[atom]   = 1.0+tf.exp(-self.p['lp3']*Delta_lp)
 
       self.EL[atom] = tf.math.divide(self.p['lp2_'+atom]*self.Delta_lp[atom],self.explp[atom],
                                      name='Elone_%s' %atom)
 
-
   def get_eover(self,atom,D,DPI):
       self.Delta_lpcorr[atom] = D - self.p['val_'+atom] - tf.math.divide(self.Delta_lp[atom],
                                 1.0+self.p['ovun3']*tf.exp(self.p['ovun4']*DPI))
@@ -730,8 +728,7 @@ class ReaxFF(object):
 
   def get_bondorder_uc(self,bd):
       self.frc[bd] = tf.where(tf.logical_or(tf.greater(self.rbd[bd],self.rc_bo[bd]),
-                                            tf.less_equal(self.rbd[bd],0.001)),
-                              tf.zeros_like(self.rbd[bd]),tf.ones_like(self.rbd[bd]))
+                                            tf.less_equal(self.rbd[bd],0.001)), 0.0,1.0)
 
       self.bodiv1[bd] = tf.math.divide(self.rbd[bd],self.p['rosi_'+bd],name='bodiv1_'+bd)
       self.bopow1[bd] = tf.pow(self.bodiv1[bd],self.p['bo2_'+bd])
@@ -1055,7 +1052,7 @@ class ReaxFF(object):
       return f
 
 
-  def get_tap(self,r,bd):
+  def get_tap(self,r,bd,di,dj):
       tp = 1.0+tf.math.divide(-35.0,tf.pow(self.vdwcut,4.0))*tf.pow(r,4.0)+ \
            tf.math.divide(84.0,tf.pow(self.vdwcut,5.0))*tf.pow(r,5.0)+ \
            tf.math.divide(-70.0,tf.pow(self.vdwcut,6.0))*tf.pow(r,6.0)+ \
@@ -1088,9 +1085,16 @@ class ReaxFF(object):
       gm3     = tf.pow(tf.math.divide(1.0,gm),3.0)
       r3      = tf.pow(rv,3.0)
       fv      = tf.where(rv>self.vdwcut,tf.zeros_like(rv),tf.ones_like(rv))
+  
+      if self.VdwFunction == 2:
+         dvi   = tf.gather_nd(self.Delta,self.vi[vb])
+         dvj   = tf.gather_nd(self.Delta,self.vj[vb])
+      else:
+         dvi   = None
+         dvj   = None
 
-      self.f_13[vb] = self.f13(rv,ai,aj)
-      self.tpv[vb] = self.get_tap(rv,vb)
+      self.f_13[vb]    = self.f13(rv,ai,aj)
+      self.tpv[vb]     = self.get_tap(rv,vb,dvi,dvj)
 
       self.expvdw1[vb] = tf.exp(0.5*self.p['alfa_'+vb]*(1.0-tf.math.divide(self.f_13[vb],
                                 2.0*self.p['rvdw_'+vb])))
@@ -1188,7 +1192,7 @@ class ReaxFF(object):
                    'tor3','tor4','cot2','coa4','ovun4',               # 
                    'ovun3','val8','val9','val10',
                    'coa3','pen2','pen3','pen4','vdw1',
-                   'cutoff','acut','hbtol'] # #
+                   'cutoff','hbtol','acut'] # # 
                    # 'trip2','trip1','trip4','trip3' ,'swa','swb'
                    # tor3,tor4>0 
 
@@ -1302,77 +1306,85 @@ class ReaxFF(object):
                 self.p[k] = tf.clip_by_value(self.v[k],0.0,690.0*self.unit)
              else:
                 self.p[k] = tf.clip_by_value(self.v[k],0.0,398.0)
+          elif k in self.clip:                 ############ clip
+             lo,hi = self.clip[k]
+             if key in self.punit:
+                lo = lo*self.unit
+                hi = hi*self.unit
+             self.p[k] = tf.clip_by_value(self.v[k],lo,hi)
+          elif key in self.clip:               ############ clip
+             lo,hi = self.clip[key]
+             if key in self.punit:
+                lo = lo*self.unit
+                hi = hi*self.unit
+             self.p[k] = tf.clip_by_value(self.v[k],lo,hi)
           elif key=='atomic':
              self.p[k] = tf.clip_by_value(self.v[k],-99.9999,999.9999)
           elif key=='acut':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0000999,0.002)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0000999,0.05)
           elif key=='hbtol':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0001,0.0002)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0000999,0.05)
           elif key=='cutoff':
              self.p[k] = tf.clip_by_value(self.v[k],0.009999,0.012)
           elif key=='ovun5':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,999.0*self.unit)
           elif key=='ovun1':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,10.0)
-          elif key in ['boc4','boc5','vdw1']:
+          elif key in ['boc4','boc5']:
              self.p[k] = tf.clip_by_value(self.v[k],0.00000001,100.0)
           elif key == 'boc3':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,50.0)
           elif key == 'gammaw':
-             self.p[k] = tf.clip_by_value(self.v[k],0.5,19.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.5,16.0)
+          elif key == 'vdw1':
+             self.p[k] = tf.clip_by_value(self.v[k],0.36,9.0)
           elif key == 'gamma':
              self.p[k] = tf.clip_by_value(self.v[k],0.001,6.0)
           elif key == 'pen1':
              self.p[k] = tf.clip_by_value(self.v[k],-69.0*self.unit,199.0*self.unit)
           elif key == 'Devdw':
-             D_= 600.0 if self.vdwnn else 0.8
-             self.p[k] = tf.clip_by_value(self.v[k],0.01*self.unit,D_*self.unit)
+             D_up= 9.9 if self.vdwnn else 6.0
+             D_lo= 0.001  if self.vdwnn else 0.001
+             self.p[k] = tf.clip_by_value(self.v[k],D_lo*self.unit,D_up*self.unit)
           elif key == 'Dehb':
              self.p[k] = tf.clip_by_value(self.v[k],-3.0*self.unit,3.0*self.unit)
           elif key == 'Desi':
-             if self.nn:
-                self.p[k] = tf.clip_by_value(self.v[k],1.0*self.unit,999.0*self.unit)
-             else:
-                self.p[k] = tf.clip_by_value(self.v[k],100.0*self.unit,999.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k],60.0*self.unit,999.0*self.unit)
           elif key== 'Depi':
              self.p[k] = tf.clip_by_value(self.v[k],0.000*self.unit,999.0*self.unit)
           elif key== 'Depp':
              self.p[k] = tf.clip_by_value(self.v[k], 0.000*self.unit,999.0*self.unit)
           elif key=='tor1':
-             self.p[k] = tf.clip_by_value(self.v[k],-39.9900,-0.0)
+             self.p[k] = tf.clip_by_value(self.v[k],-9.9900,-0.0)
           elif key== 'ovun2':
-             self.p[k] = tf.clip_by_value(self.v[k],-39.00,-0.01)
+             self.p[k] = tf.clip_by_value(self.v[k],-50.00,-0.01)
           elif key in ['tor3','val7']:
-             if key in self.punit:
-                self.p[k] = tf.clip_by_value(self.v[k],0.00,48.0*self.unit)
-             else:
-                self.p[k] = tf.clip_by_value(self.v[k],0.00,48.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.00,10.0)
           elif key in ['tor2','tor4']:
              self.p[k] = tf.clip_by_value(self.v[k],0.00,28.0)
           elif key in ['coa1','cot1']:
              self.p[k] = tf.clip_by_value(self.v[k],-99.9900*self.unit,-0.0*self.unit)
           elif key in ['V1','V2','V3']:
-             self.p[k] = tf.clip_by_value(self.v[k],-99.9900*self.unit,990.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k],-99.9900*self.unit,290.0*self.unit)
           elif key in ['bo1','bo3','bo5']:
              if self.nn:
                 if key =='bo1':
-                   self.p[k] = tf.clip_by_value(self.v[k],-0.35,-0.15)
+                   self.p[k] = tf.clip_by_value(self.v[k],-0.1,-0.005)
                 elif key =='bo3':
-                   self.p[k] = tf.clip_by_value(self.v[k],-0.35,-0.1)
+                   self.p[k] = tf.clip_by_value(self.v[k],-0.2,-0.005)
                 elif key =='bo5':
-                   self.p[k] = tf.clip_by_value(self.v[k],-0.35,-0.05)
+                   self.p[k] = tf.clip_by_value(self.v[k],-0.25,-0.005)
                 # self.p[k] = tf.clip_by_value(self.v[k],-0.35,-0.25)
              else:
                 self.p[k] = tf.clip_by_value(self.v[k],-50.0,-0.0001)
           elif key in ['bo2','bo4','bo6']:
              if self.nn:
                 if key =='bo2':
-                   self.p[k] = tf.clip_by_value(self.v[k],3.5,8.5)
+                   self.p[k] = tf.clip_by_value(self.v[k],3.0,10.0)
                 elif key =='bo4':
-                   self.p[k] = tf.clip_by_value(self.v[k],1.5,8.5)
+                   self.p[k] = tf.clip_by_value(self.v[k],3.0,10.0)
                 elif key =='bo6':
-                   self.p[k] = tf.clip_by_value(self.v[k],1.0,8.5)
-                # self.p[k] = tf.clip_by_value(self.v[k],4.5,5.0)
+                   self.p[k] = tf.clip_by_value(self.v[k],3.0,10.0)
              else:
                 self.p[k] = tf.clip_by_value(self.v[k],0.0,50.0)
           elif key == 'boc1':
@@ -1380,7 +1392,10 @@ class ReaxFF(object):
           elif key == 'boc2':
              self.p[k] = tf.clip_by_value(self.v[k],0.001,50.0)
           elif key == 'alfa':
-             self.p[k] = tf.clip_by_value(self.v[k],0.001,20.0)
+             if self.vdwnn:
+                self.p[k] = tf.clip_by_value(self.v[k],3.0,16.0)
+             else:
+                self.p[k] = tf.clip_by_value(self.v[k],0.1,16.0)
           elif key== 'lp1':
              self.p[k] = tf.clip_by_value(self.v[k],1.0,50.0)
           elif key== 'lp2':
@@ -1390,7 +1405,7 @@ class ReaxFF(object):
           elif key == 'be1':
              self.p[k] = tf.clip_by_value(self.v[k],-6.0,6.0)
           elif key == 'be2':
-             self.p[k] = tf.clip_by_value(self.v[k],0.01,19.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.1,9.0)
           elif key in ['coa2','coa3']:
              self.p[k] = tf.clip_by_value(self.v[k],-10.0,30.0)
           elif key == 'rosi':
@@ -1400,24 +1415,29 @@ class ReaxFF(object):
                 bd = b[0] +'-' +b[0]
              self.p[k] = tf.clip_by_value(self.v[k],0.90*self.re[bd],1.1*self.re[bd]) # 0.95 1.2
           elif key == 'ropi':
-             self.p[k] = tf.clip_by_value(self.v[k],0.85*self.p_['rosi_'+vn[1]],0.95*self.p_['rosi_'+vn[1]])
+             self.p[k] = tf.clip_by_value(self.v[k],0.87*self.p_['rosi_'+vn[1]],0.93*self.p_['rosi_'+vn[1]])
           elif key == 'ropp':
-             self.p[k] = tf.clip_by_value(self.v[k],0.75*self.p_['ropi_'+vn[1]],0.85*self.p_['ropi_'+vn[1]])
+             self.p[k] = tf.clip_by_value(self.v[k],0.87*self.p_['ropi_'+vn[1]],0.93*self.p_['ropi_'+vn[1]])
           elif key == 'rohb':
              self.p[k] = tf.clip_by_value(self.v[k],1.5,3.6)
           elif key == 'rvdw':
-             rvdw_ = 1.3*self.p_['rosi_'+vn[1]]
-             self.p[k] = tf.clip_by_value(self.v[k],rvdw_,2.5*self.p_['rosi_'+vn[1]])
+             rvdw_ =  self.p_['rosi_'+vn[1]]
+             self.p[k] = tf.clip_by_value(self.v[k],rvdw_,3.0*self.p_['rosi_'+vn[1]])
           elif key=='val5':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,20.0)
           elif key=='val3':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,8.0)
           elif key=='val1':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0,200.0*self.unit)
+             bd_ = vn[1]
+             a   = bd_.split('-')
+             if a[1]!='H':
+                self.p[k] = tf.clip_by_value(self.v[k],1.0,200.0*self.unit)
+             else:
+                self.p[k] = self.v[k]
           elif key=='val2':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0,8.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.2,8.0)
           elif key=='val9':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0,8.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.3,6.0)
           elif key=='val':
              sp = vn[1]
              self.p[k] = tf.clip_by_value(self.v[k],0.2,8.0)
@@ -1481,8 +1501,6 @@ class ReaxFF(object):
              self.p[k] = self.v[k]
              
       self.botol       = 0.01*self.p['cutoff']
-      self.atol        = self.p['acut']
-      self.hbtol       = self.p['hbtol']
       self.checkp()
       self.get_rcbo()
 
@@ -1653,7 +1671,7 @@ class ReaxFF(object):
              # print('-  dict %s is repeated, delteting ...' %tor3)
              torp.remove(tor3)  
       return torp 
-      
+
 
   def get_rcbo(self):
       self.rc_bo = {}
@@ -1732,7 +1750,6 @@ class ReaxFF(object):
       #     self.p[k] = self.v[k]
       # self.write_lib(libfile=self.libfile.split('.')[0]) 
 
-
   def reset(self,opt=[],libfile=None):
       if self.InitCheck:
          self.p_,self.m_ = self.ic.check(self.p_)
@@ -1748,7 +1765,6 @@ class ReaxFF(object):
       self.build_graph() 
       self.feed_dict = self.feed_data()   
 
-
   def run(self,learning_rate=1.0e-4,method='AdamOptimizer',
                step=2000,print_step=10,writelib=100,close_session=True):
       if not self.initialized:
@@ -1760,8 +1776,8 @@ class ReaxFF(object):
       libfile = self.libfile.split('.')[0]
 
       for i in range(step+1):
-          loss,latol,ME_,accu,accs,_ = self.sess.run([self.Loss,
-                                                      self.loss_atol,
+          loss,lpenalty,ME_,accu,accs,_ = self.sess.run([self.Loss,
+                                                      self.loss_penalty,
                                                       self.ME,
                                                       self.accuracy,
                                                       self.accur,
@@ -1777,16 +1793,20 @@ class ReaxFF(object):
              if close_session:
                 self.logger.info('NAN error encountered at step %d loss is %f.' %(i,loss/self.nframe))
                 # send_msg('NAN error encountered at step %d loss is %f.' %(i,loss/self.nframe))
-                tf.compat.v1.reset_default_graph()
-                self.sess.close()
-                return 9999999.9,-999.0,accMax,i,0.0
+                # tf.compat.v1.reset_default_graph()
+                # self.sess.close()
+                loss_ = 9999999.9 
+                accu  = -1.0
+                zpe   = {}
+                break
+                # return 9999999.9,-999.0,accMax,i,0.0
              else:
                 break
 
           if self.optmol:
-             loss_ = loss - latol - ME_*self.lambda_me
+             loss_ = loss - lpenalty - ME_*self.lambda_me
           else:
-             loss_ = loss - latol
+             loss_ = loss - lpenalty
 
           if i%print_step==0:
              current = time.time()
@@ -1797,7 +1817,7 @@ class ReaxFF(object):
                  acc += key+': %6.4f ' %accs[key]
 
              self.logger.info('-  step: %d sqe: %6.4f accs: %f %s spv: %6.4f me: %6.4f time: %6.4f' %(i,
-                             loss_,accu,acc,latol,ME_,elapsed_time))
+                             loss_,accu,acc,lpenalty,ME_,elapsed_time))
              self.time = current
 
           if i%writelib==0 or i==step:
@@ -1819,6 +1839,7 @@ class ReaxFF(object):
              print('-  Convergence Occurred, job compeleted.')
              break
       
+      self.get_pentalty()
       self.loss_ = loss_ if not (np.isnan(loss) or np.isinf(loss)) else 9999999.9
       if self.loss_ < 9999999.0:
          self.write_lib(libfile=libfile,loss=loss_)
@@ -1829,6 +1850,7 @@ class ReaxFF(object):
          return loss_,accu,accMax,i,zpe
 
 
+
   def feed_data(self,indexs=None):
       feed_dict = {}
       for mol in self.mols:
@@ -1839,6 +1861,7 @@ class ReaxFF(object):
              feed_dict[self.rbd[bd]] = self.lk.rbd[bd]
           if self.nvb[bd]>0:
              feed_dict[self.rv[bd]]  = self.lk.rv[bd]
+
              if self.optword.find('nocoul')<0:
                 feed_dict[self.qij[bd]] = self.lk.qij[bd]
                 # print(self.lk.vr[bd])
@@ -1862,17 +1885,14 @@ class ReaxFF(object):
              feed_dict[self.hbthe[hb]] = self.lk.hbthe[hb]
       return feed_dict
 
-
   def calculate_energy(self):
       energy = self.get_value(self.E[mol])
       return energy
 
-
   def calculate_forces(self):
       forces = 0.0
       return forces
 
-
   def get_value(self,var):
       if self.interactive:
          if type(var).__name__=='dict':
@@ -1891,7 +1911,6 @@ class ReaxFF(object):
             v = self.sess.run(var,feed_dict=self.feed_dict)
       return v
 
-
   def get_gradient(self,var,gvar):
       gd = tf.gradients(ys=var,xs=gvar)
       if gd[0] is None:
@@ -1904,7 +1923,6 @@ class ReaxFF(object):
             g = self.sess.run(grad,feed_dict=self.feed_dict)
       return g
 
-
   def get_all_gradient(self):
       tlist = tf.compat.v1.trainable_variables()
       grads = tf.gradients(ys=self.Loss, xs=tlist)
@@ -1917,7 +1935,6 @@ class ReaxFF(object):
       g,t = self.sess.run([grad,tl],feed_dict=self.feed_dict)
       return g,t,tn
 
-
   def write_lib(self,libfile='ffield',loss=None):
       p_   = self.sess.run(self.p)
       self.p_ = {}
@@ -1943,9 +1960,11 @@ class ReaxFF(object):
          # print(' * save parameters to file ...')
          fj = open(libfile+'.json','w')
          j = {'p':self.p_,'m':None,
+              'BOFunction':0,
               'EnergyFunction':0,
-              'MessageFunction':1, 
-              'messages':1,
+              'MessageFunction':0, 
+              'VdwFunction':0, 
+              'messages':0,
               'bo_layer':None,
               'mf_layer':None,
               'be_layer':None,
@@ -1962,15 +1981,16 @@ class ReaxFF(object):
                    zpe=self.zpe_,libfile=libfile,
                    loss=loss)
 
-
   def read_lib(self):
       if self.libfile.endswith('.json'):
          with open(self.libfile,'r') as lf:
               j = js.load(lf)
          self.p_  = j['p']
          self.m_  = j['m']
+         self.BOFunction_      = j['BOFunction']
          self.EnergyFunction_  = j['EnergyFunction'] 
          self.MessageFunction_ = j['MessageFunction']
+         self.VdwFunction_     = j['VdwFunction']
          self.MolEnergy_       = j['MolEnergy']
          self.bo_layer_        = j['bo_layer']
          self.mf_layer_        = j['mf_layer']
@@ -1987,7 +2007,6 @@ class ReaxFF(object):
          rcut,rcuta,re = None,None,None
       return rcut,rcuta,re
 
-
   def plot(self):
       ''' plot out some data'''
       for mol in self.mols:
@@ -2001,7 +2020,6 @@ class ReaxFF(object):
           plt.savefig('e_%s.eps' %mol) 
           plt.close()
 
-
   def plot_result(self,step,E,dfte):
       if not exists('results'):
          makedirs('results')
@@ -2029,6 +2047,100 @@ class ReaxFF(object):
              plt.savefig('results/result_%s_%s.pdf' %(mol,step)) # transparent=True
           plt.close()
 
+  def supervise(self):
+      ''' supervised learning term'''
+      penalty = 0.0
+      log_    = -9.21044036697651
+      self.penalty_ang,self.penalty_bop,self.penalty_bo = {},{},{}
+      self.penalty_rcut = {}
+      self.penalty_vdw  = {}
+
+      for bd in self.bonds: 
+          atomi,atomj = bd.split('-') 
+          bdr = atomj + '-' + atomi
+          if self.nbd[bd]>0:
+             if bd in self.bore:
+                bore_ = self.bore[bd]
+             else:
+                bore_ = self.bore['others']
+             fao = tf.where(tf.greater(self.rbd[bd],self.rcuta[bd]),
+                            tf.ones_like(self.rbd[bd]),tf.zeros_like(self.rbd[bd]))
+             self.penalty_ang[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu(self.bo0[bd]*fao-self.atol))
+             penalty = tf.add(self.penalty_ang[bd],penalty)
+
+             fbo = tf.where(tf.greater(self.rbd[bd],self.rcut[bd]),
+                            tf.ones_like(self.rbd[bd]),tf.zeros_like(self.rbd[bd]))
+             self.penalty_bop[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu(self.bop[bd]*fbo-self.botol))
+             penalty = tf.add(self.penalty_bop[bd],penalty)
+
+             # fe  = tf.where(tf.less_equal(self.rbd[bd],self.re[bd]),
+             #                tf.ones_like(self.rbd[bd]),tf.zeros_like(self.rbd[bd]))
+             # self.penalty_bo[bd] = tf.reduce_sum(input_tensor=tf.nn.relu((bore_-self.bo0[bd]))*fe)
+             
+             # penalty = tf.add(self.penalty_bop[bd],penalty)
+             # penalty = tf.add(tf.nn.relu(self.rc_bo[bd]-self.rcut[bd]),penalty)
+             if self.fixrcbo:  ### this term will let bo1,bo2,bo3,bo4 in a reasonable range
+                rcut_si = tf.square(self.rc_bo[bd]-self.rcut[bd])
+             else:
+                rcut_si = tf.nn.relu(self.rc_bo[bd]-self.rcut[bd])
+      
+             rr      = log_/self.p['bo3_'+bd] 
+             rc_bopi = self.p['ropi_'+bd]*tf.pow(log_/self.p['bo3_'+bd],1.0/self.p['bo4_'+bd])
+             rcut_pi = tf.nn.relu(rc_bopi-self.rcut[bd])
+
+             rr      = log_/self.p['bo5_'+bd] 
+             rc_bopp = self.p['ropp_'+bd]*tf.pow(log_/self.p['bo5_'+bd],1.0/self.p['bo6_'+bd])
+             rcut_pp = tf.nn.relu(rc_bopp-self.rcut[bd])
+
+             self.penalty_rcut[bd] = rcut_si + rcut_pi + rcut_pp
+             penalty= tf.add(self.penalty_rcut[bd]*self.lambda_bd,penalty)
+
+             if self.spv_vdw and self.nvb[bd]>0:          ### let VDW interaction in some range
+                self.penalty_vdw[bd] = tf.constant(0.0)
+                if (bd in self.vup) or (bdr in self.vup):
+                   bd_ = bd if bd in self.vup else bdr
+                   for vup_ in self.vup[bd_]:
+                       r_,ev_u = vup_  
+                       fu      = tf.where(tf.greater(self.rv[bd],r_),1.0,0.0) ##### 
+                       pen_v   = tf.reduce_sum(input_tensor=tf.nn.relu((self.EVDW[bd] - ev_u)*fu))
+                       self.penalty_vdw[bd] = self.penalty_vdw[bd] + pen_v  
+                       # penalty  = tf.add(self.penalty_bo_nn[bd]*self.lambda_bd,penalty) 
+
+                if (bd in self.vlo) or (bdr in self.vlo):
+                   bd_ = bd if bd in self.vlo else bdr
+                   for vlo_ in self.vlo[bd_]:
+                       r_,ev_l = vlo_
+                       fl      = tf.where(tf.less(self.rv[bd],r_),1.0,0.0) #####
+                       pen_v   = tf.reduce_sum(input_tensor=tf.nn.relu((ev_l - self.EVDW[bd])*fl))
+                       self.penalty_vdw[bd] = self.penalty_vdw[bd] + pen_v
+                penalty  = tf.add(self.penalty_vdw[bd]*self.lambda_bd,penalty)
+      return penalty*self.lambda_bd
+
+  def get_pentalty(self):
+      (penalty_bop,penalty_bo,penalty_ang,penalty_rcut,
+       rc_bo,penalty_vdw) = self.sess.run([self.penalty_bop,self.penalty_bo,self.penalty_ang,
+                               self.penalty_rcut,self.rc_bo,
+                               self.penalty_vdw], 
+                                                         feed_dict=self.feed_dict)
+
+      print('\n------------------------------------------------------------------------')
+      print('-                                                                      -')
+      print('-                         Penalty Information                          -')
+      print('-                                                                      -')
+      print('------------------------------------------------------------------------\n')
+      for bd in self.bonds:
+          if bd in penalty_bop:
+             print('Unc-Bond-Order Penalty of                          {:5s}: {:f}'.format(bd,penalty_bop[bd]))
+          if bd in penalty_bo:
+             print('Bond-Order Penalty of                              {:5s}: {:f}'.format(bd,penalty_bo[bd]))
+          if bd in penalty_ang:
+             print('BO angle cutoff Penalty of                         {:5s}: {:f}'.format(bd,penalty_ang[bd]))
+          if bd in penalty_rcut:
+             print('Differency between rcut and rcut-bo Penalty of     {:5s}: {:6.4f} {:6.4f} {:6.4f}'.format(bd,
+                                            penalty_rcut[bd],rc_bo[bd],self.rcut[bd]))
+          if bd in penalty_vdw:
+             print('Vdw fluctuation penalty of                         {:5s}: {:6.4f}'.format(bd,penalty_vdw[bd]))
+      print('\n')
 
   def close(self):
       print('-  Job compeleted.')
@@ -2108,9 +2220,7 @@ def test_reax(dataset=None,batch_size=200,dft='siesta'):
     molecules = rn.initialize()
     rn.session(learning_rate=1.0e-10,method='AdamOptimizer')
     
-    
     for mol in dataset:
-
         RE['Total-Energy'] = rn.get_value(rn.E[mol])
         RE['ebond']        = rn.get_value(rn.ebond[mol])
         RE['elonepair']    = rn.get_value(rn.elone[mol])
@@ -2167,34 +2277,63 @@ def test_reax(dataset=None,batch_size=200,dft='siesta'):
             GE['Total-Energy'].append(e_)
 
         GE['Total-Energy'] = np.array(GE['Total-Energy']) + zpe
-        
-
         x = np.linspace(0,len(RE['Total-Energy']),len(RE['Total-Energy']))
+    
+    # plot_test_result(RE,GE)
+    plt.figure(figsize=(15,10))
+    plt.subplot(3,3,1)
+    plt.plot(RE['ebond'],alpha=0.8,linestyle='-',marker='^',color='r',label='reax-Ebond')
+    plt.plot(GE['ebond'], alpha=0.8, linestyle='-', marker='o', color='b', label='gulp-Ebond')
+    plt.legend(loc='best', edgecolor='yellowgreen')
 
-        for key in GE:
-            plt.figure()             # test
-            plt.ylabel('%s (eV)' %key)
-            plt.xlabel('Step')
-            
-            # err = GE[key] - RE[key]
-            plt.plot(x,GE[key],linestyle='-',marker='o',markerfacecolor='none',
-                     markeredgewidth=1,markeredgecolor='k',
-                     ms=7,c='k',alpha=0.01,label='GULP')
-            plt.plot(x,RE[key],linestyle='-',marker='^',markerfacecolor='none',  # snow
-                     markeredgewidth=1,markeredgecolor='b',
-                     ms=7,c='b',alpha=0.01,label='I-ReaxFF')
+    plt.subplot(3,3,2)
+    plt.plot(RE['eover'],alpha=0.8,linestyle='-',marker='^',color='r',label='reax-Eover')
+    plt.plot(GE['eover'], alpha=0.8, linestyle='-', marker='o', color='b', label='gulp-Eover')
+    plt.legend(loc='best', edgecolor='yellowgreen')
 
-            # plt.errorbar(x,RE[key],yerr=err,
-            #              fmt='-.s',ecolor='r',color='r',ms=4,mfc='none',mec='blue',
-            #              elinewidth=2,capsize=2,label='I-ReaxFF')
+    plt.subplot(3,3,3)
+    plt.plot(RE['eunder'],alpha=0.8,linestyle='-',marker='^',color='r',label='reax-Eunder')
+    plt.plot(GE['eunder'], alpha=0.8, linestyle='-', marker='o', color='b', label='gulp-Eunder')
+    plt.legend(loc='best', edgecolor='yellowgreen')
 
-            plt.legend(loc='best',edgecolor='yellowgreen')
-            plt.savefig('compare_%s_%s.eps' %(mol,key),transparent=True) 
-            plt.close() 
-        # system('epstopdf compare_%s.eps' %key) 
+    plt.subplot(3,3,4)
+    plt.plot(RE['eangle'],alpha=0.8,linestyle='-',marker='^',color='r',label='reax-Eang')
+    plt.plot(GE['eangle'], alpha=0.8, linestyle='-', marker='o', color='b', label='gulp-Eang')
+    plt.legend(loc='best', edgecolor='yellowgreen')
+
+    plt.subplot(3,3,5)
+    plt.plot(RE['ecoul'],alpha=0.8,linestyle='-',marker='^',color='r',label='reax-Ecoul')
+    plt.plot(GE['ecoul'], alpha=0.8, linestyle='-', marker='o', color='b', label='gulp-Ecoul')
+    plt.legend(loc='best', edgecolor='yellowgreen')
+
+    plt.subplot(3,3,6)
+    plt.plot(RE['etorsion'],alpha=0.8,linestyle='-',marker='^',color='r',label='reax-Etor')
+    plt.plot(GE['etorsion'], alpha=0.8, linestyle='-', marker='o', color='b', label='gulp-Etor')
+    plt.legend(loc='best', edgecolor='yellowgreen')
+
+    # Ebv = np.array(Ev) + np.array(Eb) + np.array(Eo) + np.array(Eu)
+    plt.subplot(3,3,7)
+    plt.plot(RE['ehb'],alpha=0.8,linestyle='-',marker='^',color='r',label='reax-Ehb')
+    plt.plot(GE['ehb'], alpha=0.8, linestyle='-', marker='o', color='b', label='gulp-Ehb')
+    plt.legend(loc='best', edgecolor='yellowgreen')
+
+    plt.subplot(3,3,8)
+    plt.plot(RE['evdw'],alpha=0.8,linestyle='-',marker='^',color='r',label='reax-Evdw')
+    plt.plot(GE['evdw'], alpha=0.8, linestyle='-', marker='o', color='b', label='gulp-Evdw')
+    plt.legend(loc='best', edgecolor='yellowgreen')
+
+    plt.subplot(3,3,9)
+    plt.plot(RE['Total-Energy'],alpha=0.8,linestyle='-',marker='^',color='r',label='reax-Total Energy')
+    plt.plot(GE['Total-Energy'], alpha=0.8, linestyle='-', marker='o', color='b', label='gulp-Total Energy')
+    plt.legend(loc='best', edgecolor='yellowgreen')
+    plt.savefig('test_reax_energies.pdf')
+    # system('epstopdf compare_%s.eps' %key) 
 
     rn.sess.close()
+    rn.close()
 
 
 if __name__ == '__main__':
    main()
+   
+   
