@@ -707,7 +707,7 @@ class reax_data(object):
       ang_ind,self.ang_i,self.ang_j,self.ang_k = {},[],[],[]
       for i in range(self.natom): # atomic energy of i
           for n_j,j in enumerate(self.atable[i]):   
-              if j==i:
+              if j==i or self.atom_name[j]=='H':
                  continue
               for n_k,k in enumerate(self.atable[j]):
                   if k != i and k!=j:
@@ -746,10 +746,10 @@ class reax_data(object):
       tor_ind,self.tor_i,self.tor_j,self.tor_k,self.tor_l = {},[],[],[],[]
       for i in range(self.natom): # atomic energy of i
           for n_j,j in enumerate(self.atable[i]):   
-              if j==i:
+              if j==i or self.atom_name[j]=='H':
                  continue
               for n_k,k in enumerate(self.atable[j]):
-                  if k == i or k==j:
+                  if k == i or k==j or self.atom_name[j]=='H':
                      continue
                   for n_l,l in enumerate(self.atable[k]):
                       if l==k or l==j or l==i:
