@@ -18,7 +18,7 @@ line_spec.append(['ovun2','val3','n.u.','valboc','val5','n.u.','n.u.','n.u.'])
 
 line_bond = []
 line_bond.append(['Desi','Depi','Depp','be1','bo5','corr13','bo6','ovun1'])
-line_bond.append(['be2','bo3','bo4','n.u.','bo1','bo2','ovcorr','be0'])
+line_bond.append(['be2','bo3','bo4','n.u.','bo1','bo2','ovcorr','n.u.'])
 
 line_offd = ['Devdw','rvdw','alfa','rosi','ropi','ropp']
 line_ang  = ['theta0','val1','val2','coa1','val7','pen1','val4']
@@ -46,7 +46,10 @@ def read_lib(p={},zpe=False,libfile='ffield',
           lin = lines[0].split()
           for i in range(int(len(lin)/2-1)):
               k = lin[i*2+1]
-              zpe_[k] = float(lin[i*2])
+              try:
+                 zpe_[k] = float(lin[i*2])
+              except ValueError:
+                 zpe_[k] = 0.0
 
        for i in range(npar):
            pn = p_name[i]
@@ -300,3 +303,10 @@ def write_lib(p,spec,bonds,offd,angs,tors,hbs,zpe=None,libfile='irnn.lib',
         print(txt,file=flib)
     flib.close()
 
+def write_gulp_lib(libfile='reax_mpnn.lib'):
+    ''' write the force field parameters and weight and bias of neural networks to GULP library file '''
+    glib = open(libfile,'w')
+
+    glib.close()
+
+
